@@ -20,8 +20,10 @@
 #include	"task.h"
 #include	"stream.h"
 #include	"change_endian.h"
+#include	"machine/msequence.h"
 #include	"init.h"
 #include	"u_file.h"
+#include	"utils.h"
 
 
 
@@ -32,6 +34,7 @@ char * msg;
 extern char * command_arg;
 int ret;
 
+	semlock_mode = SLM_REALTIME;
 	set_fd_max(1024);
 	command_arg = argv[0];
 	check_endian();
