@@ -1468,4 +1468,22 @@ MAP * m;
 }
 
 
+BIB_LIST *
+gv_get_biblist(L_CHAR * url)
+{
+URL u;
+RESOURCE * r;
+BIB_LIST * ret;
+	get_url2(&u,url);
+	r = search_resource_by_entry(&u);
+	if ( r == 0 ) {
+		ret = 0;
+		goto end;
+	}
+	ret = r->h.bib;
+end:
+	free_url(&u);
+	return ret;
+}
+
 
