@@ -22,6 +22,7 @@
 #include	"memory_debug.h"
 #include	"xlerror.h"
 #include	"resource.h"
+#include	"init.h"
 #include	"utils.h"
 #include	"xl.h"
 #include	"task.h"
@@ -32,7 +33,6 @@
 #include	"save_global.h"
 #define HANDLING_MAX		2
 
-
 extern SEM res_lock;
 extern GV_FUNC gv_new_table[RT_MAX];
 extern XLISP_ENV * gv_resource_env[RT_MAX];
@@ -88,6 +88,9 @@ void
 init_load_resource()
 {
 
+	init_load_meta();
+
+
 	memset(&res_extend,0,sizeof(SYS_QUEUE));
 	res_extend.flags = QF_STACK;
 	res_extend.gc_func = gc_lrw;
@@ -147,6 +150,7 @@ init_load_resource()
 	create_task(resource_disposer_task,0,PRI_FETCH);
 
 	resource_global();
+
 }
 
 
