@@ -700,7 +700,6 @@ int tim;
 	xli->a_type = XLA_SELF;
 	setup_i(xli);
 
-ss_printf("EXEC DIRTY\n");
 	for ( ; ; ) {
 		tim = get_xltime();
 		for ( ; ; ) {
@@ -711,7 +710,7 @@ ss_printf("EXEC DIRTY\n");
 			d = dirty_list;
 			dirty_list = d->next;
 			unlock_task(omx_lock,"_exec_dirty");
-		
+ss_printf("EXEC DIRTY === INSERT DIRTY\n");
 			wf_insert_dirty_rect(d->r,
 				&d->rct,WFF_LUSTER_DIRTY,120,10);
 			d_f_ree(d);
@@ -742,6 +741,7 @@ insert_dirty(RESOURCE * r,GB_RECT * rct)
 OMX_DIRTY * d;
 int t;
 
+ss_printf("OLD_MATRIX INSERT DIRTY\n");
 	lock_task(omx_lock);
 	d = d_alloc(sizeof(*d));
 	d->r = r;
@@ -795,18 +795,18 @@ MATRIX_TOKEN * n2;
 		r = ow2->r;
 
 		ir.tl.x = n->dim_code[1] & 
-			(- (1<<(n->dim_code[0]*m->dim_divide[0]
+			(- (((INTEGER64)1)<<(n->dim_code[0]*m->dim_divide[0]
 			+ m->block_size[0])));
 		ir.tl.y = n->dim_code[2] &
-			(- (1<<(n->dim_code[0]*m->dim_divide[1]
+			(- (((INTEGER64)1)<<(n->dim_code[0]*m->dim_divide[1]
 			+ m->block_size[1])));
 		ir.br.x = ir.tl.x +
-				(1<<(n->dim_code[0] * m->dim_divide[0] +
+				(((INTEGER64)1)<<(n->dim_code[0] * m->dim_divide[0] +
 					m->block_size[0]));
 		if ( ir.br.x > m->pixel_size[0] )
 			ir.br.x = m->pixel_size[0];
 		ir.br.y = ir.tl.y +
-				(1<<(n->dim_code[0] * m->dim_divide[1] +
+				(((INTEGER64)1)<<(n->dim_code[0] * m->dim_divide[1] +
 					m->block_size[1]));
 		if ( ir.br.y > m->pixel_size[1] )
 			ir.br.y = m->pixel_size[1];
