@@ -26,16 +26,6 @@
 void ov_default();
 
 
-void
-set_weight(WIN_FLAME * wf)
-{
-	if ( wf->flags & WFF_LUSTER )
-		wf->draw->c.weight = W_E2D_LUSTER;
-	else if ( wf->flags & WFF_POLY )
-		wf->draw->c.weight = W_E2D_POLY;
-	else	wf->draw->c.weight = W_E2D_PLOT;
-}
-
 int
 e2d_new(
 	WIN_FLAME_TABLE * tbl,
@@ -89,6 +79,10 @@ int ret;
 	wf->mh = copy_mh(mh);
 	wf->v_reso = v_reso;
 	wf->flags = WFF_ACTIVE|WFF_LUSTER|WFF_DIRTY;
+
+	wf->data_request = 0;
+	wf->dr_time = 0;
+	wf->dr_interval = 1;
 /*
 	draw->c.weight = 0;
 */
@@ -123,6 +117,7 @@ int ret;
 	wf->out_base_time = 0;
 
 	wf->insert_time = get_xltime();
+	
 
 	_wakeup_wf(gf,wf);
 end:
@@ -425,9 +420,50 @@ e2d_change(
 	WIN_FLAME * wf)
 {
 int i;
+int y;
+I_RECT rct;
+unsigned long * pixels, * ptr;
+int width,x_offset,w_len;
 	switch ( wf->tbl->wf_type ) {
 	case WFT_STD:
 		break;
+	case WFT_SMALL:
+		pixels = d_alloc(gf->win_width * gf->win_height * sizeof(long));
+		ptr = pixels;
+		for ( i =  gf->win_width * gf->win_height ; i < 0 ; i -- )
+			*ptr++ = C_TRANSPARENT;
+		rct = wf->small_rct;
+		if ( rct.tl.x < 0 )
+			rct.tl.x = 0;
+		else if ( rct.tl.x >= gf->win_width )
+			goto next;
+		if ( rct.br.x >= gf->win_width )
+			rct.br.x = gf->win_width;
+		else if ( rct.br.x < 0 )
+			goto next;
+		if ( rct.tl.y < 0 )
+			rct.tl.y = 0;
+		else if ( rct.tl.y >= gf->win_height )
+			goto next;
+		if ( rct.br.y >= gf->win_height )
+			rct.br.y = gf->win_height;
+		else if ( rct.br.y < 0 )
+			goto next;
+		x_offset = rct.tl.x - wf->small_rct.tl.x;
+		w_len = rct.br.x - rct.tl.x;
+		width = wf->small_rct.br.x - wf->small_rct.tl.x;
+		for ( y = rct.tl.y ; y < rct.br.y ; y ++ ) {
+			if( y < 0 || y >= gf->win_height )
+				continue;
+			memcpy(&pixels[y*gf->win_width + rct.tl.x],
+				&wf->pixels[(y - wf->small_rct.tl.y)* width
+						+x_offset],
+				w_len * sizeof(long));
+		}
+	next:
+		d_f_ree(wf->pixels);
+		wf->pixels = pixels;
+		break;
 	case WFT_PLOT:
 		wf->pixels = d_alloc(gf->win_width * gf->win_height 
 				* sizeof(long));
@@ -437,8 +473,8 @@ int i;
 	default:
 		er_panic("e2d_change not support");
 	}
-	set_weight(wf);
 	wf->tbl = tbl;
+	set_weight(gf,wf);
 	return 0;
 }
 
@@ -509,7 +545,6 @@ SURP_SET surp;
 int id;
 int ci_ret;
 
-
 	if ( r->tl.x < 0 ) {
 		if ( r->br.x <= 0 )
 			return -1;
@@ -620,10 +655,13 @@ int ci_ret;
 
 	mh = add_mh(&gf->flame_base_display_map,wf->mh);
 
+
 /*
 map_test(pt_list,pt_reso,w,w*h,"e2d-1");
 ss_printf(">>e2d-1\n");
 */
+//ss_printf("MAP LEN 1 = %i\n",map_history_length(mh));
+
 	if ( wf->flags & WFF_LUSTER )
 		map_from_flame(mh,pt_list,pt_reso,w*h);
 /*
@@ -637,14 +675,12 @@ map_test_trace(mh);
 	id = wf->id;
 	wf_unlock(gf);
 
-
 	ci_ret = cache_image(
 		gf,
 		wf->id,wf->draw,pt_list,pt_reso,buf,
 		w*h,r,mh,&surp,1,
 		WFF_PLOT_DIRTY|WFF_LUSTER_DIRTY);
 
-
 	wf_lock(gf);
 	_wf = _get_wf_ptr(gf,id);
 	if ( _wf != wf )
@@ -771,13 +807,12 @@ restart:
 		rr.br.x = rr.tl.x + rp.w;
 		rr.br.y = rr.tl.y + rp.h;
 /*
-ss_printf("** %i(%x) ** xx %i %i %i %i %i %i\n",get_tid(),wf->flags,
+ss_printf("** %i(%x) ** yy %i %i %i %i %i %i\n",get_tid(),wf->flags,
 xx,yy,t,b,l,r);
 */
 		switch ( e2d_redraw_rect(gf,wf,&rr,&pri_cnt) ) {
 		case 0:
 
-
 			ret = 1;
 		case -1:
 			break;
@@ -816,8 +851,9 @@ int flags;
 	ret = 0;
 
 	wf_lock(gf);
-	set_weight(wf);
+	set_weight(gf,wf);
 	flags = wf->flags;
+
 	if ( wf->flags & (WFF_POLY_DIRTY|WFF_LUSTER_DIRTY) ) {
 		wf->flags &= ~(WFF_POLY_DIRTY|WFF_LUSTER_DIRTY);
 		wf_unlock(gf);
@@ -925,17 +961,16 @@ int i;
 	return 0;
 }
 
-int
-e2d_resize(GBVIEW_FLAME * gf,WIN_FLAME * wf)
-{
-int w,h;
-int i;
-	w = gf->win_width;
-	h = gf->win_height;
 
-	wf->pixels = d_re_alloc(wf->pixels,w*h*sizeof(long));
-	for ( i = 0 ; i < w*h ; i ++ )
-		wf->pixels[i] = C_DIRTY|C_TRANSPARENT;
+
+int
+e2d_resize(GBVIEW_FLAME * gf,WIN_FLAME * wf,int old_w,int old_h)
+{
+unsigned long * pixels;
+	pixels = wf_get_resize_data(wf->pixels,old_w,old_h,gf->win_width,gf->win_height,C_DIRTY);
+
+	d_f_ree(wf->pixels);
+	wf->pixels = pixels;;
 	if ( wf->flags & WFF_LUSTER )
 		wf->flags |= WFF_LUSTER_DIRTY;
 	if ( wf->flags & WFF_POLY )
@@ -968,3 +1003,22 @@ int i;
 	return 0;
 }
 
+void
+e2d_setup_drawrect(GBVIEW_FLAME * gf,WIN_FLAME * wf,GB_RECT r,float rate)
+{
+I_RECT rct;
+	rct = get_drawrect(gf,r,rate);
+	if ( rct.br.x - rct.tl.x < gf->win_width/2 &&
+			rct.br.y - rct.tl.y < gf->win_height/2 ) {
+		wf->flags |= WFF_TO_SMALL;
+		wf->flags &= ~WFF_TO_STD;
+	}
+	else {
+		wf->flags &= ~WFF_TO_SMALL;
+		wf->flags |= WFF_TO_STD;
+	}
+}
+
+
+
+
