@@ -33,6 +33,7 @@
 #include	"save_global.h"
 #include	"xl.h"
 #include	"radar.h"
+#include	"h_layering.h"
 
 #define CURRENT_SWITCH_TIMERAG	2
 #define ZONBIE_TIMEOUT	120
@@ -67,7 +68,6 @@ void _indicate_flame_lock(int*);
 void _call_flame_change_base_event(GBVIEW_FLAME * gf);
 GBVIEW_PLOT_ELEMENT *  _wf_plot_element(GBVIEW_FLAME * gf,WF_ID id);
 
-
 void
 init_win_flame()
 {
@@ -75,6 +75,8 @@ init_win_flame()
 	
 	sg("GBVIEW_FLAME*","gbview_flame_list",&gbview_flame_list);
 
+	init_h_layering();
+
 	gbview_flame_lock = new_lock(LL_WF);
 /* init GBVIEW_FLAME
 	gf->seq = 1;
@@ -87,23 +89,26 @@ init_win_flame()
 	create_task(change_task,0,PRI_DRAW_EXIT);
 	
 	upper_layer_lock_tracer = _indicate_flame_lock;
+
+
 }
 
 void
 see_lock_d(char * msg,GBVIEW_FLAME * gf,LOCK_D * d,int * ind)
 {
-	if ( d->file == 0 )
+	if ( d->_file == 0 )
 		return;
 	if ( debug_th_timeout_check(d->time) < 0 )
 		return;
 	if ( *ind == 0 )
 		printf("LOCK INFO ====\n");
 	*ind = 1;
-	printf("LOCK_D (%s %x) %isec :: %s %i\n",
+	printf("LOCK_D (%s %x) %isec :: %s %i == %i\n",
 		msg,(int)gf,
 		get_xltime() - d->time,
-		d->file,
-		d->line);	
+		d->_file,
+		d->_line,
+		d->lock_cnt);	
 }
 
 void
@@ -209,16 +214,23 @@ ch_lock_init(LOCK_D * d)
 void
 _ch_lock(LOCK_D *lock_bit,char * file,int line)
 {
-	for ( ; lock_bit->file ; ) {
+HTID tid;
+	tid = _get_tid();
+	if ( tid.valid == lock_bit->tid.valid && tid.tid == lock_bit->tid.tid ) {
+		lock_bit->lock_cnt ++;
+		return;
+	}
+	for ( ; lock_bit->_file ; ) {
 		_i_sleep_task((int)lock_bit,gbview_flame_lock,file,line);
 		lock_task(gbview_flame_lock);
 	}
 	if ( file == 0 )
 		er_panic("_ch_lock");
-	lock_bit->file = file;
-	lock_bit->line = line;
-	lock_bit->tid = _get_tid();
+	lock_bit->_file = file;
+	lock_bit->_line = line;
+	lock_bit->tid = tid;
 	lock_bit->time = get_xltime();
+	lock_bit->lock_cnt = 1;
 }
 
 void
@@ -236,10 +248,15 @@ int pri;
 void
 _ch_unlock(LOCK_D * lock_bit)
 {
-	if ( lock_bit->file == 0 )
+	if ( lock_bit->_file == 0 )
 		er_panic("wf_unlock");
-	lock_bit->file = 0;
-	lock_bit->line = 0;
+	lock_bit->lock_cnt --;
+	if ( lock_bit->lock_cnt < 0 )
+		er_panic("wf_unlock(2)");
+	if ( lock_bit->lock_cnt )
+		return;
+	lock_bit->_file = 0;
+	lock_bit->_line = 0;
 	lock_bit->tid = invalid_htid();
 	wakeup_task((int)lock_bit);
 }
@@ -262,7 +279,7 @@ int pri;
 int
 _ch_locked(LOCK_D * d)
 {
-	if ( d->file )
+	if ( d->_file )
 		return 1;
 	return 0;
 }
@@ -271,11 +288,12 @@ void
 ch_sleep_task(int key,LOCK_D * lock_bit,void (*f)(),void * wk)
 {
 	lock_task(gbview_flame_lock);
-	if ( lock_bit->file == 0 )
+	if ( lock_bit->_file == 0 )
 		er_panic("ch_sleep_task");
-	lock_bit->file = 0;
-	lock_bit->line = 0;
+	lock_bit->_file = 0;
+	lock_bit->_line = 0;
 	lock_bit->tid = invalid_htid();
+	lock_bit->lock_cnt = 0;
 	wakeup_task((int)lock_bit);
 	if ( f )
 		(*f)(wk);
@@ -342,7 +360,7 @@ XL_INTERPRETER * xli;
 	f = (F2OUTLIST*)d;
 
 	log_printf(LOG_MESSAGE,LOG_LAYER_GB,0,
-		"EXIT %ls\n",
+		"EXIT-1 %ls\n",
 		get_url_str2(&f->wf->draw->h.entry));
 
 	exit_win_flame(f->gf,f->wf);
@@ -577,7 +595,7 @@ ok:
 //				exit_lock_resource(wf->draw,0);
 
 			log_printf(LOG_MESSAGE,LOG_LAYER_GB,0,
-				"EXIT %ls\n",
+				"EXIT-2 %ls\n",
 				get_url_str2(&wf->draw->h.entry));
 
 				exit_win_flame(gf,wf);
@@ -677,13 +695,18 @@ int ret;
 void
 _xx_wakeup_wf(GBVIEW_FLAME * gf,WIN_FLAME * wf,char * __f,int __l)
 {
-
-//ss_printf("_xx_wakeup_wf %s::%i\n",__f,__l);
-
+/*
+ss_printf("_xx_wakeup_wf %s::%i %i %i\n",__f,__l,gf->run_req_task_nos,gf->load_task_max);
+*/
 	wf->flags |= WFF_DRAW_REQUEST;
 	gf->req_task_nos ++;
-	if ( gf->run_req_task_nos < LOAD_TASK_MAX )
+	if ( gf->load_task_max <= 0 )
+		gf->load_task_max ++;
+	wakeup_task((int)gf);
+	if ( gf->run_req_task_nos <= gf->load_task_max ) {
+		gf->run_req_task_nos ++;
 		create_task(load_task,(int)gf,PRI_START_DRAW);
+	}
 }
 
 void
@@ -784,15 +807,15 @@ wf_active_check_2(GBVIEW_FLAME * gf,WF_ID id)
 {
 WIN_FLAME * wf;
 int ret;
-/*
+
 	wf_lock(gf);
 	ZONBIE_CHECK(gf,wf_unlock,-1);
-*/
+
 	wf = _get_wf_ptr(gf,id);
 	if ( wf == 0 )
 		ret = 1;
 	else	ret = _wf_active_check(wf);
-//	wf_unlock(gf);
+	wf_unlock(gf);
 	return ret;
 }
 
@@ -858,38 +881,76 @@ WIN_FLAME * wf;
 	return ret;
 }
 
+GV_RESOURCE_LIST *
+_search_resource_list(GBVIEW_FLAME * gf,RESOURCE *r,L_CHAR * target,int create_flag)
+{
+GV_RESOURCE_LIST * ret,** rp;
+	for ( rp = &gf->resource_list ; *rp ; rp = &(*rp)->next ) {
+		ret = *rp;
+		if ( r ) {
+			if ( ret->draw == 0 ) {
+				if ( l_strcmp(ret->entry,get_url_str2(&r->h.entry)) == 0 ) {
+					ret->draw = r;
+					goto ok;
+				}
+			}
+			else if ( ret->draw == r )
+				goto ok;
+		}
+		else {
+			if ( ret->draw == 0 ) {
+				if ( l_strcmp(ret->entry,target) == 0 )
+					goto ok;
+			}
+			else if ( l_strcmp(get_url_str2(&ret->draw->h.entry),target) == 0 )
+				goto ok;
+		}
+	}
+	if ( create_flag ) {
+		ret = d_alloc(sizeof(*ret));
+		memset(ret,0,sizeof(*ret));
+		ret->draw = r;
+		ret->entry = ll_copy_str(target);
+		ret->next = gf->resource_list;
+		gf->resource_list = ret;
+		return ret;
+	}
+	return 0;
+ok:
+		*rp = ret->next;
+		ret->next = gf->resource_list;
+		gf->resource_list = ret;
+		return ret;
+}
+
+GV_RESOURCE_LIST *
+search_resource_list(GBVIEW_FLAME * gf,RESOURCE *r,L_CHAR * target,int create_flag)
+{
+GV_RESOURCE_LIST * ret;
+	wf_lock(gf);
+	ZONBIE_CHECK(gf,wf_unlock,0);
+	ret = _search_resource_list(gf,r,target,create_flag);
+	wf_unlock(gf);
+	return ret;
+}
 
 void
 _setup_wf_flags(GBVIEW_FLAME * gf,WIN_FLAME * wf)
 {
 GV_RESOURCE_LIST * rl;
-	for ( rl = gf->resource_list ; rl ; rl = rl->next ){
-		if ( url_cmp(&rl->entry_url,&wf->draw->h.entry) )
-			continue;
-		wf->flags &= ~WFF_COPY_FLAGS;
-		wf->flags |= (WFF_COPY_FLAGS & rl->flags);
+	rl = _search_resource_list(gf,wf->draw,0,0);
+	if ( rl == 0 )
 		return;
-	}
-	return;
+	wf->flags &= ~WFF_COPY_FLAGS;
+	wf->flags |= (WFF_COPY_FLAGS & rl->flags);
 }
 
 void
 _backup_wf_flags(GBVIEW_FLAME * gf,WIN_FLAME * wf)
 {
 GV_RESOURCE_LIST * rl;
-	for ( rl = gf->resource_list ; rl ; rl = rl->next ) {
-		if ( url_cmp(&rl->entry_url,&wf->draw->h.entry) )
-			continue;
-		goto ok;
-	}
-	if ( (wf->flags & WFF_COPY_FLAGS) == 0 )
-		return;
-	rl = d_alloc(sizeof(*rl));
-	copy_url(&rl->entry_url,&wf->draw->h.entry);
-	rl->flags = 0;
-	rl->next = gf->resource_list;
-	gf->resource_list = rl;
-ok:
+
+	rl = _search_resource_list(gf,wf->draw,0,1);
 	rl->flags &= ~WFF_COPY_FLAGS;
 	rl->flags |= (wf->flags & WFF_COPY_FLAGS);
 }
@@ -901,7 +962,8 @@ GV_RESOURCE_LIST * rl;
 	for ( ; gf->resource_list ; ) {
 		rl = gf->resource_list;
 		gf->resource_list = rl->next;
-		free_url(&rl->entry_url);
+		if ( rl->entry )
+			d_f_ree(rl->entry);
 		d_f_ree(rl);
 	}
 }
@@ -939,6 +1001,7 @@ RE_DATA * dp;
 	else d_f_ree(dp);
 }
 
+
 typedef struct cb_arg {
 	int	d_type;
 	void	(*d_event)(struct gbview_flame*,int,void*,void*);
@@ -1020,6 +1083,19 @@ call_flame_insert_event(GBVIEW_FLAME * gf)
 	wf_unlock(gf);
 }
 
+void
+call_flame_status_event(GBVIEW_FLAME * gf)
+{
+void (*event)(struct gbview_flame*,int,void*,void*);
+void *arg;
+	wf_lock(gf);
+	event = gf->event;
+	arg = gf->event_user_arg;
+	wf_unlock(gf);
+	if ( event )
+		(*event)(gf,ET_STATUS,0,arg);
+}
+
 void
 insert_redraw_r(GBVIEW_FLAME * gf,VRECT * r)
 {
@@ -1987,12 +2063,13 @@ I_RECT rr;
 }
 
 void
-wf_wakeup(GBVIEW_FLAME * gf)
+xx_wf_wakeup(GBVIEW_FLAME * gf,char * __f,int __l)
 {
 WIN_FLAME * wf;
 	wf_lock(gf);
+	ZONBIE_CHECK_VOID(gf,wf_unlock(gf))
 	for ( wf = gf->flame ; wf ; wf = wf->next ) {
-		_wakeup_wf(gf,wf);
+		_xx_wakeup_wf(gf,wf,__f,__l);
 	}
 	wf_unlock(gf);
 }
@@ -2158,6 +2235,8 @@ int ret;
 	default:
 		er_panic("wf_move");
 	}
+	if ( gf->history_enable )
+		gf->history_flag = 1;
 	wf_unlock(gf);
 	new_tick((void(*)(int))_wf_set_redraw,0,(int)gf);
 	return ret;
@@ -2192,6 +2271,7 @@ MAP_HISTORY * mh;
 	map_from_flame(mh,ptr,reso,ptr_len);
 	*name = get_url_str2(&gf->flame_base->draw->h.entry);
 	*rid = gf->flame_base->draw->h.no;
+	free_mh(mh);
 
 	wf_unlock(gf);
 }
@@ -2233,6 +2313,7 @@ int * index_x, * index_y;
 int len;
 int ret;
 VRECT rr;
+
 	wf_lock(gf);
 	ZONBIE_CHECK(gf,wf_unlock,0);
 
@@ -2264,6 +2345,8 @@ VRECT rr;
 
 	ret = _get_env_seq(gf);
 	gf->move_flag = 1;
+	if ( gf->history_enable )
+		gf->history_flag = 1;
 	wf_unlock(gf);
 	assert_ls(gf,LSF_ZOOM);
 	_wakeup_win_manage(gf->radar_ptr);
@@ -2282,37 +2365,104 @@ VRECT rr;
 }
 
 
+unsigned long *
+wf_get_resize_data(unsigned long * src,int old_w,int old_h,int w,int h,unsigned long dirty)
+{
+int y,x;
+int xx;
+int h_ofs,w_ofs;
+int y_ofs;
+unsigned long * pixels,*dest;
+
+
+	h_ofs = (h - old_h)/2;
+	w_ofs = (w - old_w)/2;
+	pixels = d_alloc(w*h*sizeof(long));
+
+	dest = pixels;
+	if ( h_ofs + old_h < h ) {
+		for ( y = 0 ; y < h_ofs ; y ++ )
+			for ( x = 0 ; x < w ; x ++ )
+				*dest++ = C_TRANSPARENT|dirty;
+		for ( ; y < h_ofs + old_h ; y ++ ) {
+			y_ofs = (y-h_ofs)*old_w;
+			for ( x = 0 ; x < w ; x ++ , dest ++ ) {
+				xx = x - w_ofs;
+				if ( xx < 0 )
+					*dest = C_TRANSPARENT|dirty;
+				else if ( xx >= old_w )
+					*dest = C_TRANSPARENT|dirty;
+				else {
+					*dest = src[y_ofs + xx];
+				}
+			}
+		}
+		for ( ; y < h ; y ++ )
+			for ( x = 0 ; x < w ; x ++ )
+				*dest ++ = C_TRANSPARENT|dirty;
+	}
+	else {
+		for ( y = 0 ; y < h ; y ++ ) {
+			y_ofs = (y-h_ofs)*old_w;
+			for ( x = 0 ; x < w ; x ++ , dest ++ ) {
+				xx = x - w_ofs;
+				if ( xx < 0 )
+					*dest = C_TRANSPARENT|dirty;
+				else if ( xx >= old_w )
+					*dest = C_TRANSPARENT|dirty;
+				else {
+					*dest = src[y_ofs + xx];
+				}
+			}
+		}
+	}
+	return pixels;
+}
+
 
 void
-_change_redraw(GBVIEW_FLAME * gf)
+_change_redraw(GBVIEW_FLAME * gf,int old_width,int old_height)
 {
 int w,h;
 int i;
+unsigned long * redraw_plane;
+VRECT r;
 	w = gf->win_width;
 	h = gf->win_height;
 
+	if ( gf->redraw_plane ) {
+		redraw_plane = wf_get_resize_data(gf->redraw_plane,old_width,old_height,w,h,0);
+		d_f_ree(gf->redraw_plane);
+		gf->redraw_plane = redraw_plane;
+	}
+	else {
+		gf->redraw_plane = d_alloc(sizeof(long)*w*h);
+		for ( i = 0 ; i < w*h ; i ++ )
+			gf->redraw_plane[i] = 0;
+	}
 	gf->redraw = d_re_alloc(gf->redraw,w*h);
-	gf->redraw_plane = d_re_alloc(gf->redraw_plane,w*h*sizeof(long));
 	for ( i = 0 ; i < w*h ; i ++ )
 		gf->redraw[i] = 0;
-	gf->win_height = h;
-	gf->win_width = w;
+	r.tl.x = r.tl.y = 0;
+	r.br.x = w;
+	r.br.y = h;
+	insert_redraw_r(gf,&r);
 }
 
 void
-_wf_change_size(GBVIEW_FLAME * gf)
+_wf_change_size(GBVIEW_FLAME * gf,int old_width,int old_height)
 {
 WIN_FLAME * wf;
 
 	_ex_lock(gf);
 
 	for ( wf = gf->flame ; wf ; wf = wf->next ) {
-		win_flame_resize(gf,wf);
+		win_flame_resize(gf,wf,old_width,old_height);
 		wf->flags |= WFF_RESIZE;
 		_wakeup_wf(gf,wf);
 	}
 	_calc_pitch(gf,1);
-	_change_redraw(gf);
+	_change_redraw(gf,old_width,old_height);
 
 	new_tick((void(*)(int))_wf_set_redraw,0,(int)gf);
 
@@ -2470,6 +2620,8 @@ int ret;
 	a.d_y = y;
 	a.d_x_w = w;
 	a.d_y_h = h;
+
+
 	for ( wf = gf->flame ; wf ; wf = wf->next ) {
 		if ( wf->flags & (WFF_FREE|WFF_HIDE) )
 			continue;
@@ -2557,7 +2709,6 @@ int ret;
 	_ex_lock(gf);
 
 	ret = __wf_redraw(gf,p,x,y,w,h);
-
 	_ex_unlock(gf);
 	wf_unlock(gf);
 	return ret;
@@ -2994,6 +3145,8 @@ int ses;
 		}
 	}
 	_calc_pitch(gf,1);
+	if ( gf->history_enable )
+		gf->history_flag = 1;
 	_all_dirty_out(gf);
 end1:
 	assert_ls(gf,LSF_ZOOM);
@@ -3041,6 +3194,7 @@ I_RECT tar;
 		sw.si = si;
 		si->pixels = 0;
 		select_resource(wf->draw,&sw);
+		free_mh(sw.mh);
 		if ( si->pixels ) {
 			si->t = 0.4;
 			si->rev_t = 1-si->t;
@@ -3174,7 +3328,88 @@ int i;
 		wf->flags |= WFF_PLOT;
 }
 
+void
+wf_check_small(GBVIEW_FLAME * gf,WIN_FLAME * wf)
+{
+SURP_SET surp;
+MAP_HISTORY * mh;
+GB_RECT * ret;
+int n;
+int i;
+GB_RECT rct;
+	set_surp(&surp,&wf->draw->h.minrect,1);
+	mh = add_mh(&gf->flame_base_display_map,wf->mh);
+	map_from_resource(mh,surp.ptr,surp.reso,SURP_NOS);
+	ret = get_surp_rect(&n,&surp,0);
+	free_mh(mh);
+	if ( ret == 0 )
+		return;
+	rct.tl.x = rct.tl.y = 0;
+	rct.br.x = rct.br.y = -1;
+	for ( i = 0 ; i < n ; i ++ )
+		add_rect(&rct,&ret[i]);
+	d_f_ree(ret);
 
+	if( gf->flame_base_display_map.type == MHT_PP )
+		wf_setup_drawrect(gf,wf,rct,1.5);
+	else	wf_setup_drawrect(gf,wf,rct,1.05);
+}
+
+void
+wf_check_small_2(GBVIEW_LAYER_STATUS * ls,GBVIEW_FLAME * gf,WIN_FLAME * wf)
+{
+SURP_SET surp;
+MAP_HISTORY * mh;
+GB_RECT * ret;
+int n;
+int i;
+GB_RECT rct;
+GB_POINT ptr[2],center;
+REAL1 reso[2];
+REAL1 distance;
+	set_surp(&surp,&wf->draw->h.minrect,1);
+	mh = add_mh(&gf->flame_base_display_map,wf->mh);
+	map_from_resource(mh,surp.ptr,surp.reso,SURP_NOS);
+	ret = get_surp_rect(&n,&surp,0);
+	if ( ret == 0 ) {
+		free_mh(mh);
+		return;
+	}
+	rct.tl.x = rct.tl.y = 0;
+	rct.br.x = rct.br.y = -1;
+	for ( i = 0 ; i < n ; i ++ )
+		add_rect(&rct,&ret[i]);
+	d_f_ree(ret);
+
+	if( gf->flame_base_display_map.type == MHT_PP )
+		ls->small_rect = get_drawrect(gf,rct,1.5);
+	else	ls->small_rect = get_drawrect(gf,rct,1.05);
+	
+	center.x = ((REAL1)gf->win_width)/2;
+	center.y = ((REAL1)gf->win_height)/2;
+
+	distance = 10;
+	for ( ; ; ) {
+		ptr[0] = center;
+		ptr[1] = center;
+		ptr[1].y += distance;
+		reso[0] = reso[1] = 1;
+		map_from_flame(mh,ptr,reso,2);
+		if ( reso[0] < 0 ) {
+			ls->distance = -1;
+			break;
+		}
+		if ( reso[1] < 0 ) {
+			distance = distance / 2;
+			continue;
+		}
+		ls->center[0] = ptr[0];
+		ls->center[1] = ptr[1];
+		ls->distance = distance;
+		break;
+	}
+	free_mh(mh);
+}
 
 void
 wf_check_resource(GBVIEW_FLAME * gf)
@@ -3205,7 +3440,6 @@ int i;
 			wf->flags |= WFF_POLY;
 		if ( c.opt[RT_DRAW_GB] & (DGF_INFO|DGF_MARK) )
 			wf->flags |= WFF_PLOT;
-
 	}
 	wf_unlock(gf);
 }
@@ -3491,6 +3725,8 @@ WIN_FLAME * wf;
 	_calc_pitch(gf,0);
 	gf->flame_base_rotate -= get_radian(gf,from,to);
 	_calc_pitch(gf,1);
+	if ( gf->history_enable )
+		gf->history_flag = 1;
 
 	ret = _get_env_seq(gf);
 	wf_unlock(gf);
@@ -3552,7 +3788,7 @@ RESOURCE * map;
 WFID_INCLUDE_RESOURCE * wir,* p;
 WIN_FLAME * wf;
 int mask;
-
+int i;
 	mask = 0;
 	wir = wf_search_wfid_include_resource(r);
 	for ( p = wir ; p ; p = p->next ) {
@@ -3570,18 +3806,23 @@ int mask;
 		set_surp(&ss,d_rect,1);
 		draw = wf_get_mh(p->gf,p->id,&mh);
 		if ( draw == 0 )
-			continue;
+			goto next2;
 		map = get_map_between2(r,draw);
 		if ( map == 0 )
-			continue;
+			goto next2;
 		map_conv_forward(map,ss.ptr,ss.reso,SURP_NOS);
+		for ( i = 0 ; i < SURP_NOS ; i ++ )
+			if ( ss.reso[i] < 0 ) {
+				set_surp(&ss,&map->h.minrect,1);
+				break;
+			}
 		map_from_resource(mh,ss.ptr,ss.reso,SURP_NOS);
 		memset(ss.group,0,sizeof(ss.group));
 		_rct = get_surp_rect(&nos,&ss,0);
 		if ( nos > 1 )
 			er_panic("wf_insert_dirty_rect");
 		if ( nos == 0 )
-			continue;
+			goto next2;
 		rct = *_rct;
 
 		rct.tl.x -= ofs_w;
@@ -3589,9 +3830,10 @@ int mask;
 		rct.br.x += ofs_w;
 		rct.br.y += ofs_h;
 		win_flame_dirty(p->gf,p->id,&rct,flags & mask,0);
-
-		free_mh(mh);
+//	next1:
 		d_f_ree(_rct);
+	next2:
+		free_mh(mh);
 	}
 	free_wfid_include_resource(wir);
 }
@@ -3708,7 +3950,7 @@ end1:
 }
 
 void
-wf_status(GBVIEW_FLAME * gf,GBVIEW_STATUS * e)
+wf_status_flags(GBVIEW_FLAME * gf,GBVIEW_STATUS * e,int req_flags)
 {
 WIN_FLAME * wf;
 int n;
@@ -3746,7 +3988,7 @@ GBVIEW_LAYER_STATUS * ls, ** lsp;
 	}
 	else	e->current = 0;
 
-	if ( gf->req_task_nos || gf->run_req_task_nos )
+	if ( gf->req_task_nos || gf->req_task_nos2 )
 		e->loading_status.draw = 1;
 	if ( gf->exit_task_wp )
 		e->loading_status.exit = 1;
@@ -3765,6 +4007,8 @@ GBVIEW_LAYER_STATUS * ls, ** lsp;
 	e->loading_status.indicate = gf->indicate_task_active;
 	wf_indicate_unlock(gf);
 	e->loading_status.raw_image_request = gf->raw_image_request;
+	e->history_flag = gf->history_flag;
+	e->history_enable = gf->history_enable;
 
 	e->flags = SF_BASE;
 
@@ -3796,6 +4040,7 @@ GBVIEW_LAYER_STATUS * ls, ** lsp;
 				continue;
 			ls = d_alloc(sizeof(*ls));
 			ls->id = wf->id;
+			ls->mh_length = map_history_length(wf->mh);
 			ls->entry_url = ll_copy_str(
 				get_url_str2(&wf->draw->h.entry));
 			ls->target_url = ll_copy_str(
@@ -3804,6 +4049,9 @@ GBVIEW_LAYER_STATUS * ls, ** lsp;
 			ls->flags = wf->flags&WFF_COPY_FLAGS&(~WFF_BASE_CURRENT);
 			if ( gf->flame_base == wf )
 				ls->flags |= WFF_BASE_CURRENT;
+			ls->hide_time = wf->hide_time;
+			if ( req_flags & SF_LAYER_DETAILS )
+				wf_check_small_2(ls,gf,wf);
 			ls->next = 0;
 			*lsp = ls;
 			lsp = &ls->next;
@@ -3820,6 +4068,12 @@ GBVIEW_LAYER_STATUS * ls, ** lsp;
 	unlock_task(in_lock,"wf_status");
 }
 
+void
+wf_status(GBVIEW_FLAME * gf,GBVIEW_STATUS * e)
+{
+	wf_status_flags(gf,e,0);
+}
+
 int
 change_status_rule(int * old_st,int new_st)
 {
@@ -3942,6 +4196,7 @@ wf_set_status(GBVIEW_FLAME * gf,GBVIEW_STATUS * e)
 int ret;
 int change,st_change;
 int shift;
+int old_width,old_height;
 	wf_lock(gf);
 	ZONBIE_CHECK(gf,wf_unlock,0)
 	change = 0;
@@ -3953,6 +4208,9 @@ int shift;
 			st_change = gf->status;
 		else	st_change = 0;
 	}
+
+	old_width = gf->win_width;
+	old_height = gf->win_height;
 	if ( e->flags & SF_WIDTH ) {
 		if ( gf->win_width != e->width )
 			change = 1;
@@ -3971,10 +4229,16 @@ int shift;
 	}
 	if ( e->flags & SF_DEFAULT_WS )
 		gf->default_ws = e->default_ws;
+	if ( e->flags & SF_HIS_ENABLE )
+		gf->history_enable = e->history_enable;
+	if ( e->flags & SF_HIS_FLAG )
+		gf->history_flag = e->history_flag;
 
 	if ( change )
-		_wf_change_size(gf);
-	ret = _get_env_seq(gf);
+		_wf_change_size(gf,old_width,old_height);
+	if ( e->flags & SF_DONT_SEQ )
+		ret = 0;
+	else	ret = _get_env_seq(gf);
 	wf_unlock(gf);
 
 	if ( gf->radar_ptr == 0 && (e->flags & SF_RADAR) )
@@ -4129,18 +4393,37 @@ unsigned int ret;
 	return ret;
 }
 
+void
+dr_tick(int d)
+{
+GBVIEW_FLAME * gf;
+	gf = (GBVIEW_FLAME*)d;
+	wf_wakeup(gf);
+}
+
 void
 wf_set_data_request(GBVIEW_FLAME * gf,WF_ID wfid,int r)
 {
 WIN_FLAME * wf;
 	wf_lock(gf);
 	wf = _get_wf_ptr(gf,wfid);
-	if ( wf )
+	if ( wf ) {
+		if ( r ) {
+			wf->dr_time = get_xltime() + wf->dr_interval;
+			new_tick((void(*)())dr_tick,-wf->dr_interval-1,(int)gf);
+			wf->dr_interval *= 2;
+		}
+		else {
+			wf->dr_interval = DR_INTERVAL_MIN;
+			wf->dr_time = 0;
+		}
 		wf->data_request = r;
+	}
 	wf_unlock(gf);
 }
 
 
+
 int
 wf_set_overlay(GBVIEW_FLAME* gf,L_CHAR * url,int flags,int mask)
 {
@@ -4154,6 +4437,20 @@ GV_RESOURCE_LIST * ri;
 	ret = 0;
 	wf_lock(gf);
 	ZONBIE_CHECK(gf,wf_unlock,-1);
+	if ( url == 0 ) {
+		if ( mask & WFF_BASE_LOCK ) {
+			f = 0;
+			for ( wf2 = gf->flame ; wf2 ; wf2 = wf2->next ) {
+				if ( wf2->flags & WFF_BASE_LOCK )
+					f = 1;
+				wf2->flags &= ~WFF_BASE_LOCK;
+			}
+			_call_re_base_lock(gf,ET_LOCK_BASE);
+			for ( ri = gf->resource_list ; ri ; ri = ri->next )
+				ri->flags &= ~WFF_BASE_LOCK;
+		}
+		goto end2;
+	}
 	get_url2(&u,url);
 	for ( wf = gf->flame ; wf ; wf = wf->next ) {
 		if ( wf->flags & WFF_FREE )
@@ -4162,6 +4459,20 @@ GV_RESOURCE_LIST * ri;
 			continue;
 		goto ok;
 	}
+	if ( flags & WFF_BASE_LOCK ) {
+		f = 0;
+		for ( wf2 = gf->flame ; wf2 ; wf2 = wf2->next ) {
+			if ( wf2->flags & WFF_BASE_LOCK )
+				f = 1;
+			wf2->flags &= ~WFF_BASE_LOCK;
+		}
+		_call_re_base_lock(gf,ET_LOCK_BASE);
+		for ( ri = gf->resource_list ; ri ; ri = ri->next )
+			ri->flags &= ~WFF_BASE_LOCK;
+	}
+	ri = _search_resource_list(gf,0,url,1);
+	ri->flags &= ~(WFF_COPY_FLAGS&mask);
+	ri->flags |= flags & WFF_COPY_FLAGS;
 	ret = -1;
 	goto end;
 ok:
@@ -4174,6 +4485,9 @@ ok:
 		rr.tl.x = rr.tl.y = 0;
 		rr.br.x = gf->win_width;
 		rr.br.y = gf->win_height;
+		
+		if ( (flags & WFF_HIDE_TIME_DISABLE) == 0 )
+			wf->hide_time = get_xltime();
 		if ( flags ) {
 			wf->flags |= WFF_HIDE;
 			set_redraw_rect(gf,&rr);
@@ -4184,6 +4498,7 @@ ok:
 			_win_flame_dirty(gf,wf,&rr,WFF_DIRTY,0);
 			force_change_task();
 		}
+		_call_flame_change_base_event(gf);
 	}
 	if ( wf->flags & WFF_BASE_LOCK ) {
 		for ( wf2 = gf->flame ; wf2 ; wf2 = wf2->next ) {
@@ -4199,7 +4514,23 @@ ok:
 	}
 end:
 	free_url(&u);
+end2:
 	wf_unlock(gf);
 	return ret;
 }
 
+int wf_get_flags(GBVIEW_FLAME * gf,WF_ID wfid)
+{
+int ret;
+WIN_FLAME * wf;
+	wf_lock(gf);
+	ZONBIE_CHECK(gf,wf_unlock,-1);
+	wf = _get_wf_ptr(gf,wfid);
+	if ( wf == 0 )
+		ret = -1;
+	else	ret = wf->flags;
+	wf_unlock(gf);
+	return ret;
+}
+
+
