@@ -18,13 +18,12 @@
 
 #include	"mx.h"
 
-void init_gmxGetPoint(XLISP_ENV * ,XLISP_ENV * );
-
 void
 init_gmx_function(XLISP_ENV * env0,XLISP_ENV * env1)
 {
 
-	
-	init_gmxGetPoint(env0,env1);
+	init_gmx_lib_function(env0,env1);
 
+	init_genExit(env1);
+	init_gmxGet(env0);
 }
