@@ -132,6 +132,10 @@ typedef struct d_sexp {
 
 typedef struct remote_session_opt {
 	int		timeout_count;
+	int		return_format;
+#define RFT_DONTCARE	0
+#define RFT_HTTP	1
+#define RFT_BODY	2
 } REMOTE_SESSION_OPT;
 
 typedef struct xl_result {
@@ -744,7 +748,7 @@ XL_SEXP * xl_Delete(XLISP_ENV * env,XL_SEXP * el,XL_SEXP * s);
 void init_MemoryStatistics(XLISP_ENV * env);
 void init_NaturalJoin(XLISP_ENV * env);
 void init_MoveDefine(XLISP_ENV * env);
-int njoin_cmp_field(XL_SEXP ** retp,XLISP_ENV * env,XL_SEXP * s1,XL_SEXP * s2);
+int njoin_cmp_field(XL_SEXP ** retp,XLISP_ENV * env,XL_SEXP * s1,XL_SEXP * s2,XL_FILE*,int);
 int njoin_cmp_field_list(XL_SEXP ** retp,
 	XLISP_ENV * env,
 	XL_SEXP * flist,
@@ -892,8 +896,8 @@ int open_session_http(int open_opt);
 void close_session_http(int id);
 void output_session_http(STREAM * con,char * server,int port);
 L_CHAR * get_method(L_CHAR * method);
-XL_SEXP * read_reply(STREAM * st,XL_FILE * f,int ln);
-XL_SEXP * one_session(URL * u,XL_SEXP * s,XL_FILE * f,int ln,int wait_flag);
+XL_SEXP * read_reply(STREAM * st,REMOTE_SESSION_OPT*,XL_FILE * f,int ln);
+XL_SEXP * one_session(URL * u,XL_SEXP * s,REMOTE_SESSION_OPT*,XL_FILE * f,int ln,int wait_flag);
 XL_SEXP * remote_session_http(
 	XLISP_ENV * env,
 	int id,
@@ -954,7 +958,8 @@ void init_Touch(XLISP_ENV*);
 void init_FormatEval(XLISP_ENV * env);
 void init_GetListbaseEnv(XLISP_ENV * env);
 void init_SetListbaseEnv(XLISP_ENV * env);
-
+void init_DefineEval(XLISP_ENV * env);
+void init_LaunchWWW(XLISP_ENV * env);
 
 void kill_thread(XL_SEXP * target);
 
@@ -968,6 +973,8 @@ int get_my_port();
 XL_SEXP *
 convert_code(XL_SEXP * st,LCZ_SET * set,int flags);
 char * convert_percent(char * inp,char*);
+char *
+convert_percent_reverse(char * inp);
 void * xl_search_ri_org(int * err,int *,L_CHAR * path);
 void *
 xl_get_ri(int * err,int * w,int * h,ri_que_cf func,void * cf_data,L_CHAR * path);
