@@ -24,20 +24,23 @@
 #include	"xl.h"
 #include	"xlerror.h"
 
+#include	"mx_blk_h_layering.h"
+#include	"mx_blk_primitive.h"
+#include	"mx_blk_s_array.h"
+
 /*
 #define SCRIPT_TRIGGER
 #define SCRIPT_DEBUG
 #define SCRIPT_ERR
 */
 
-
+#define TIME_DOMAIN	120
 
 typedef struct load_work {
 	int		pri;
 	int		cnt;
 } LOAD_WORK;
 
-void gc_tick_notin_tick();
 
 int NC_flag;
 
@@ -58,7 +61,7 @@ void _close_matrix(MATRIX * m);
 MATRIX *
 _open_matrix(L_CHAR * neturl,L_CHAR * filename,L_CHAR * key,void (*open_method)(),void * work);
 
-MATRIX_NODE * _get_matrix_node(int * errp,MATRIX * m,INTEGER64 * dim_code,int,INTEGER64);
+MATRIX_NODE * _get_matrix_node(int * errp,MATRIX * m,INTEGER64 * dim_code,int,INTEGER64,XL_SEXP **);
 
 void _wait_matrix_mode(MATRIX * m);
 int _set_matrix_channel_info(MATRIX * m,int ch,MATRIX_CHANNEL_INFO * inf);
@@ -199,6 +202,8 @@ init_matrix()
 	insert_matrix_type_table(&mx_type_string);
 	insert_matrix_type_table(&mx_type_string_v);
 	insert_matrix_type_table(&mx_type_sexp);
+	insert_matrix_type_table(&mx_type_RGB);
+	insert_matrix_type_table(&mx_type_RGB_v);
 
 	matrix_xl_env = new_env(gblisp_top_env0);
 
@@ -221,8 +226,13 @@ init_matrix()
 	init_mxSquash(matrix_xl_env);
 	init_mxStuffing(matrix_xl_env);
 	init_mxMax(matrix_xl_env);
+	init_mxLoadSB(matrix_xl_env);
+	init_mxSaveSB(matrix_xl_env);
 
-
+	init_mxSetSB(matrix_xl_env);
+	init_sbArray(matrix_xl_env);
+	init_sbPrimitive(matrix_xl_env);
+	init_sbHeuristicLayering(matrix_xl_env);
 	
 	new_tick(matrix_access_tick,STAT_TICK,0);
 }
@@ -284,6 +294,8 @@ mx_purge()
 {
 MATRIX_NODE * n1,*n2;
 int start_nd_cnt;
+
+ss_printf("mx_purge==============================================\n");
 	n1 = nd_root->h.prev;
 	start_nd_cnt = nd_cnt/2;
 	for ( ; n1 != nd_root && nd_cnt > start_nd_cnt ; ) {
@@ -292,6 +304,7 @@ int start_nd_cnt;
 			continue;
 		if ( n1->dirty_file_time )
 			_matrix_trigger_access(n1,MI_SAVE_TP,0);
+ss_printf("mx_purge======================================FREE %p\n",n1);
 		_free_matrix_node(n1);
 		n1 = n2;
 	}
@@ -413,7 +426,6 @@ MATRIX_TOKEN * t;
 
 		gc_pop(0,0);
 	}
-
 	close_self_interpreter();
 	release_qkey(&matrix_que,key);
 }
@@ -451,6 +463,27 @@ static unsigned int last_delete;
 				_matrix_trigger_access(n1,MI_SAVE_TP,0);
 			cnt ++;
 		}
+
+ss_printf("insert purge matrix\n");
+
+#ifdef TIME_DOMAIN
+		for ( ; n1 != nd_root ; ) {
+			n2 = n1->h.next;
+			if ( now - n1->last_access < TIME_DOMAIN ) {
+				n1 = n2;
+				continue;
+			}
+			if ( n1 == n ) {
+				n1 = n2;
+				continue;
+			}
+			if ( n1->dirty_file_time && n1->status == MS_OK )
+				_matrix_trigger_access(n1,MI_SAVE_TP,0);
+			_free_matrix_node(n1);
+			n1 = n2;
+		}
+#else	
+
 		for ( ; n1 != nd_root ; ) {
 			n2 = n1->h.next;
 			if ( n1 != n ) {
@@ -460,6 +493,7 @@ static unsigned int last_delete;
 			}
 			n1 = n2;
 		}
+#endif
 	}
 }
 
@@ -494,13 +528,15 @@ _delete_matrix_sexp(MATRIX_SEXP * d)
 void
 gc_matrix_lock()
 {
-	lock_task(matrix_gc_lock);
+	if ( lock_up_test(matrix_gc_lock) )
+		lock_task(matrix_gc_lock);
 }
 
 void
 gc_matrix_unlock()
 {
-	unlock_task(matrix_gc_lock,"gc_matrix_unlock");
+	if ( lock_up_test(matrix_gc_lock) )
+		unlock_task(matrix_gc_lock,"gc_matrix_unlock");
 }
 
 
@@ -508,6 +544,8 @@ void
 gc_matrix()
 {
 MATRIX_SEXP * d, * dn;
+	if ( ms_root.next == 0 )
+		return;
 	gc_gblisp_env(matrix_xl_env);
 	for ( d = ms_root.next ; d != &ms_root ; ) {
 		dn = d->next;
@@ -576,6 +614,7 @@ void
 _insert_matrix_open(MATRIX * m,void (*func)())
 {
 MATRIX_TOKEN * t;
+
 	t = d_alloc(sizeof(*t));
 	memset(t,0,sizeof(*t));
 	t->h.key = nl_copy_str(std_cm,"_insert_matrix_open");
@@ -706,9 +745,10 @@ _close_matrix(MATRIX * m)
 int i;
 int f;
 MATRIX_SEXP * d;
-MATRIX_NODE * n;
+MATRIX_NODE * n, * n2;
 MATRIX_TOKEN * t;
 int ret_id;
+//int pri;
 int retry_f;
 	retry_f = 0;
 	if ( m->mode == MM_CLOSE )
@@ -743,23 +783,35 @@ retry:
 			continue;
 		f = 1;
 		n = m->node_hash[i];
-		if ( (ret_id = _free_matrix_node(m->node_hash[i]) )< 0 ) {
-ss_printf("close_matirx ret_id = %i\n",ret_id);
-			if ( n->dirty_file_time && n->status == MS_OK )
-				_matrix_trigger_access(n,MI_SAVE_TP,0);
-			t = m->node_hash[i]->process_token;
-			if ( t && t->aboat_func )
-				(*t->aboat_func)(t);
+		for ( ; n ; ) {
+			n2 = n->nh_next;
+			if ( (ret_id = _free_matrix_node(n) )< 0 ) {
+ss_printf("close_matirx ret_id = %i %i\n",ret_id,matrix_que.total_cnt);
+				if ( n->dirty_file_time && n->status == MS_OK )
+					_matrix_trigger_access(n,MI_SAVE_TP,0);
+				t = n->process_token;
+				if ( t && t->aboat_func )
+					(*t->aboat_func)(t);
 /*
-			sleep_task((int)n,matrix_lock);
-			lock_task(matrix_lock);
+				sleep_task((int)n,matrix_lock);
+				lock_task(matrix_lock);
 */
-			retry_f = 1;
+				retry_f = 1;
+			}
+			n = n2;
 		}
 	}
+	if ( m->token_cnt ) {
+		retry_f = 1;
+	}
 	if ( retry_f ) {
 		unlock_task(matrix_lock,"");
 		sleep_sec(1);
+/*
+		pri = get_pri(0);
+		change_pri(0,PRI_RCACHE);
+		change_pri(0,pri);
+*/
 		lock_task(matrix_lock);
 	
 		goto retry;
@@ -1285,6 +1337,18 @@ int ret;
 	return ret;
 }
 
+
+void
+normalize_dim_code(MATRIX * m,INTEGER64 * dim)
+{
+INTEGER64 mask;
+int i;
+	for ( i = 0 ; i < m->p.dim ; i ++ ) {
+		mask = -(((INTEGER64)1)<<(m->block_size[i] + m->dim_divide[i]*dim[0]));
+		dim[i+1] = dim[i+1] & mask;
+	}
+}
+
 int 
 _cmp_dim_code(MATRIX * m,INTEGER64 * dim_code1,INTEGER64 * dim_code2)
 {
@@ -1478,6 +1542,41 @@ see_access_list(MATRIX_ACCESS_LIST * a)
 	ss_printf(")\n");
 }
 
+void
+_matrix_send_result(MATRIX_TOKEN * t)
+{
+D_SEXP * d;
+MATRIX_NODE * n;
+MATRIX_SEXP * dd1,*dd2;
+	n = t->process_node;
+	if ( n->send_data_delay == 0 )
+		return;
+	d = n->send_data_delay->data;
+	if ( t->process_node->status == MS_OK ) {
+		set_d_sexp(d,encode_matrix_node(n));
+	}
+	else {
+		set_d_sexp(d,
+			get_error(
+				0,
+				0,
+				XLE_PROTO_INV_OBJECT,
+				l_string(std_cm,"Get"),
+				n_get_string("get_matrix there is no node")));
+	}
+//	lock_task(matrix_lock);
+	dd1 = n->send_data_delay;
+	dd2 = n->send_data;
+	n->send_data_delay = 0;
+	n->send_data = 0;
+//	unlock_task(matrix_lock,"_matrix_send_result");
+
+	gc_matrix_lock();
+	_delete_matrix_sexp(dd1);
+	_delete_matrix_sexp(dd2);
+	gc_matrix_unlock();
+}
+
 
 void
 matrix_exec_cal(MATRIX_TOKEN * t)
@@ -1495,11 +1594,13 @@ MATRIX * m;
 int i;
 #endif
 
+
 check_point(t);
 
 	e = t->env->data;
 	m = t->process_node->matrix;
 
+
 #ifdef SCRIPT_DEBUG
 
 ss_printf("ENTR %x:%i(%i) = NODE=%x [%x]+%i+(%i %i %i %i -- atid=%i cal=%x)\n",t,t->access_target_id,
@@ -1576,6 +1677,7 @@ see_access_list(t->access_list);
 
 		ret = eval(e,car(t->cal_target));
 
+
 		if ( get_type(ret) != XLT_ERROR ) {
 
 #ifdef SCRIPT_DEBUG
@@ -1602,6 +1704,13 @@ ss_printf("\n");
 #endif
 		switch ( ret->err.code ) {
 		case XLE_SYSTEM_EXIT:
+			lock_task(matrix_lock);
+
+			t->process_node->scan_call[t->access_target_id/3] = ME_EXIT;
+			wakeup_task((int)&t->process_node->scan_call[t->access_target_id/3]);
+
+			unlock_task(matrix_lock,"matrix");
+
 			gc_pop(0,0);
 			return;
 		case XLE_SYSTEM_APPLICATION:
@@ -1621,6 +1730,9 @@ ss_printf("\n");
 //		log_print_sexp(LOG_ERROR,LOG_LAYER_XL,0,"matrix_exec_cal",ret,0);
 		gc_pop(0,0);
 		lock_task(matrix_lock);
+		t->process_node->scan_call[t->access_target_id/3] = ME_ERROR;
+		wakeup_task((int)&t->process_node->scan_call[t->access_target_id/3]);
+
 		if ( t->err_jump_status >= 0 ) {
 #ifdef SCRIPT_DEBUG
 ss_printf("ERST %x:%i = %i ns=%i\n",t,t->access_target_id,
@@ -1629,6 +1741,8 @@ t->process_node->status);
 see_access_list(t->access_list);
 #endif
 			t->process_node->status = t->err_jump_status;
+
+			_matrix_send_result(t);
 			n = t->process_node;
 			for ( ; n->wait_token ; ) {
 				t2 = n->wait_token;
@@ -1660,6 +1774,9 @@ see_access_list(t->access_list);
 	}
 	lock_task(matrix_lock);
 success:
+	t->process_node->scan_call[t->access_target_id/3] = 1;
+	wakeup_task((int)&t->process_node->scan_call[t->access_target_id/3]);
+
 	if ( t->normal_jump >= 0 ) {
 
 #ifdef SCRIPT_DEBUG
@@ -1680,6 +1797,7 @@ see_access_list(t->access_list);
 		t->process_node->status = t->normal_jump_status;
 		if ( t->process_node->status == MS_OK && t->process_node->matrix->p.trigger )
 			(*t->process_node->matrix->p.trigger)(TRT_TOKEN,t);
+		_matrix_send_result(t);
 		n = t->process_node;
 		for ( ; n->wait_token ; ) {
 			t2 = n->wait_token;
@@ -1697,7 +1815,6 @@ see_access_list(t->access_list);
 	}
 end:
 
-
 #ifdef SCRIPT_DEBUG
 ss_printf("EXIT %x:%i =\n",t,t->access_target_id);
 #endif
@@ -1726,6 +1843,8 @@ end1:
 	unlock_task(matrix_lock,"matrix_exec_cal");
 	if ( m->token_cnt == 0 && m->p.trigger )
 		(*m->p.trigger)(TRT_FINISH,0);
+
+
 }
 
 void
@@ -1734,6 +1853,7 @@ _matrix_token_error(MATRIX_TOKEN * t)
 	if ( t->err_jump_status >= 0 ) {
 		t->process_node->status = t->err_jump_status;
 		wakeup_task((int)t->process_node);
+		_matrix_send_result(t);
 	}
 	if ( t->err_jump >= 0 ) {
 		t->cal_target = 0;
@@ -1821,6 +1941,7 @@ MATRIX*m;
 XLISP_ENV * e;
 MATRIX_SEXP * d;
 
+
 	gc_push(0,0,"_new_matrix_token");
 	if ( n->process_token )
 		er_panic("_new_matrix_token");
@@ -2003,6 +2124,18 @@ INTEGER64 level;
 	return key % NHASH_SIZE;
 }
 
+void
+_free_nlist_dim_addr(INTEGER64 ** addr,int len)
+{
+int i;
+	if ( addr == 0 )
+		return;
+	for ( i = 0 ; i < len ; i ++ ) {
+		if ( addr[i] )
+			d_f_ree(addr[i]);
+	}
+	d_f_ree(addr);
+}
 
 int
 _free_matrix_node(MATRIX_NODE * n)
@@ -2030,6 +2163,7 @@ MATRIX_DATA_TYPE * md;
 */
 	_delete_node_ring(n);
 	*np = n->nh_next;
+	
 	d_f_ree(n->dim_code);
 	if ( n->channel ) {
 		for ( i = 0 ; i < m->p.channel_nos ; i ++ ) {
@@ -2038,12 +2172,14 @@ MATRIX_DATA_TYPE * md;
 				continue;
 			if ( n->channel[i].data )
 				(*md->free_data)(md,n->channel[i].data);
+			if ( n->channel[i].sb )
+				(*n->channel[i].sb->tbl->free_sb)(n->channel[i].sb);
 		}
 	}
 	if ( n->nlist_dim_bit_field )
 		d_f_ree(n->nlist_dim_bit_field);
 	if ( n->nlist_dim_addr )
-		d_f_ree(n->nlist_dim_addr);
+		_free_nlist_dim_addr(n->nlist_dim_addr,n->nlist_dim_addr_len);
 	d_f_ree(n);
 	return 0;
 }
@@ -2062,10 +2198,39 @@ _sync_matrix_node(MATRIX_NODE * n)
 	return 0;
 }
 
+void
+set_send_data(MATRIX_NODE * n,XL_SEXP ** retp)
+{
+D_SEXP * d;
+MATRIX_SEXP * md;
+XL_SEXP * ret;
+	if ( retp ) {
+		if ( n->send_data ) {
+			*retp = n->send_data->data;
+			return;
+		}
+		ret = new_d_sexp(&d);
+		*retp = ret;
 
+		md = d_alloc(sizeof(*md));
+		memset(md,0,sizeof(*md));
+		md->data = d;
+		md->gc_func = gc_d_sexp;
+		_insert_matrix_sexp(md);
+		n->send_data_delay = md;
+		
+		md = d_alloc(sizeof(*md));
+		memset(md,0,sizeof(*md));
+		md->data = ret;
+		md->gc_func = gc_gb_sexp;
+		_insert_matrix_sexp(md);
+		n->send_data = md;
+		*retp = ret;
+	}
+}
 
 MATRIX_NODE *
-_get_matrix_node(int * errp,MATRIX * m,INTEGER64 * dim_code,int access,INTEGER64 lev)
+_get_matrix_node(int * errp,MATRIX * m,INTEGER64 * dim_code,int access,INTEGER64 lev,XL_SEXP ** retp)
 {
 MATRIX_NODE * n;
 unsigned int key;
@@ -2097,8 +2262,6 @@ unsigned int key;
 		if ( _cmp_dim_code(m,n->dim_code,dim_code) == 0 )
 			break;
 	if ( n ) {
-
-
 		_delete_node_ring(n);
 		_insert_node_ring(n);
 		n->req_level = lev;
@@ -2107,6 +2270,7 @@ unsigned int key;
 		case MS_PROCESS:
 			if ( errp )
 				*errp = ME_PROC_NODE;
+			set_send_data(n,retp);
 			break;
 		case MS_OK:
 			break;
@@ -2158,6 +2322,8 @@ unsigned int key;
 	memset(n,0,sizeof(*n));
 	n->matrix = m;
 	n->dim_code = _copy_dim_code(m,dim_code);
+	normalize_dim_code(m,n->dim_code);
+//ss_printf("NORMALIZE %s\n",pt_dc(m,n->dim_code,PTDC_PIXEL_1));
 	n->status = MS_NOBIND;
 	wakeup_task((int)n);
 	n->nh_next = m->node_hash[key];
@@ -2170,6 +2336,7 @@ unsigned int key;
 				m->dim_bit_field);
 		n->nlist_dim_bit_field_len = m->dim_bit_field;
 	}
+	set_send_data(n,retp);
 	m->node_hash[key] = n;
 	_insert_node_ring(n);
 	_get_matrix_access(n);
@@ -2180,13 +2347,14 @@ unsigned int key;
 
 MATRIX_NODE *
 get_matrix_node(int * errp,MATRIX * m,INTEGER64 * dim_code,
-		int access,MATRIX_TOKEN * t)
+		int access,MATRIX_TOKEN * t,
+		XL_SEXP ** retp)
 {
 MATRIX_NODE * n;
 int err;
 	lock_task(matrix_lock);
 	err = 0;
-	n = _get_matrix_node(&err,m,dim_code,access,dim_code[0]);
+	n = _get_matrix_node(&err,m,dim_code,access,dim_code[0],retp);
 	if ( errp && err )
 		*errp = err;
 	if ( err ) {
@@ -2214,7 +2382,7 @@ int err;
 retry:
 	lock_task(matrix_lock);
 	err = 0;
-	n = _get_matrix_node(&err,m,dim_code,access,dim_code[0]);
+	n = _get_matrix_node(&err,m,dim_code,access,dim_code[0],0);
 	if ( n && n->status <= MS_PROCESS ) {
 
 #ifdef SCRIPT_DEBUG
@@ -2242,7 +2410,7 @@ int i;
 	for ( i = 1 ; i <= m->p.dim ; i ++ ) {
 		dd = dim_code[i] >> ((n->dim_code[0]-1)*m->dim_divide[i-1]
 				+ m->block_size[i-1]);
-		dd &= (1<<m->dim_divide[i-1])-1;
+		dd &= (((INTEGER64)1)<<m->dim_divide[i-1])-1;
 		ix = (ix << m->dim_divide[i-1]) | dd;
 	}
 	return ix;
@@ -2257,8 +2425,8 @@ INTEGER64 d;
 	ret = copy_dim_code(m,dc);
 	ret[0] --;
 	for ( i = m->p.dim - 1 ; i >= 0 ; i -- ) {
-		ret[i+1] &= -(dc[0]*m->dim_divide[i] + m->block_size[i]);
-		d = ix & ((1<<m->dim_divide[i])-1);
+		ret[i+1] &= -(((INTEGER64)1)<<(dc[0]*m->dim_divide[i] + m->block_size[i]));
+		d = ix & ((((INTEGER64)1)<<m->dim_divide[i])-1);
 		ret[i+1] |= d << (ret[0]*m->dim_divide[i] + m->block_size[i]);
 		ix >>= m->dim_divide[i];
 	}
@@ -2403,7 +2571,7 @@ INTEGER64 lev;
 	err = 0;
 	ret = 0;
 	prev = 0;
-	_ret = _get_matrix_node(&err,m,dim_code,GN_READ_ONLY,lev);
+	_ret = _get_matrix_node(&err,m,dim_code,GN_READ_ONLY,lev,0);
 	if ( err ) {
 		if ( err == ME_MATRIX_ERR )
 			goto end;
@@ -2413,7 +2581,7 @@ INTEGER64 lev;
 	_ret = 0;
 	for ( ; dim_code[0] >= lev ; dim_code[0] -- , prev = _ret ) {
 		err = 0;
-		_ret = _get_matrix_node(&err,m,dim_code,access,lev);
+		_ret = _get_matrix_node(&err,m,dim_code,access,lev,0);
 		if ( err == 0 )
 			ret = _ret;
 		else {
@@ -2480,7 +2648,7 @@ retry:
 		goto end;	
 	}
 	err = 0;
-	n = _get_matrix_node(&err,m,dim_code,access,dim_code[0]);
+	n = _get_matrix_node(&err,m,dim_code,access,dim_code[0],0);
 	if ( err == ME_MATRIX_ERR ) {
 		if ( errp )
 			*errp = err;
@@ -2518,6 +2686,11 @@ ok:
 					MD_DALLOC,
 					dim_code,
 					m->channel_info[ch].default_data);
+				if ( ret == 0 ) {
+					if ( errp )
+						*errp = ME_INDEX_OVER;
+					goto end;
+				}
 			}
 			else {
 				ret = mxt_alloc_vector(
@@ -2548,7 +2721,7 @@ end:
 }
 
 INTEGER64 *
-get_dim_code_from_sexp(XL_SEXP * s)
+get_dim_code_from_sexp(MATRIX * m,XL_SEXP * s)
 {
 INTEGER64 *ret;
 int len;
@@ -2557,12 +2730,14 @@ XL_SEXP * ss;
 	len = list_length(s);
 	if ( len < 2 )
 		return 0;
+	if ( len != m->p.dim+1 )
+		return 0;
 	ret = d_alloc(sizeof(INTEGER64)*len);
 	for ( i = 0 ; i < len ; i ++ , s = cdr(s) ) {
 		ss = car(s);
 		if ( get_type(ss) != XLT_INTEGER )
 			ret[i] = 0;
-		ret[i] = ss->integer.data;
+		else	ret[i] = ss->integer.data;
 	}
 	return ret;
 }
@@ -2725,13 +2900,63 @@ XL_SEXP * ret;
 	return ret;
 }
 
+
+
+MX_STRUCT_BLOCK * 
+_get_channel_struct_block(MATRIX_NODE * n,int ch)
+{
+MATRIX * m;
+MATRIX_DATA_TYPE * md;
+char * err_msg;
+
+	err_msg =0;
+	if  ( n->channel == 0 )
+		return 0;
+	m = n->matrix;
+	if ( ch < 0 || ch >= m->p.channel_nos )
+		return 0;
+	md = m->channel_info[ch].data_type;
+	if ( md->type != MDT_BLOCK )
+		return 0;
+	_matrix_node_channel_lock(n);
+	return n->channel[ch].sb;
+}
+
+MX_STRUCT_BLOCK * 
+get_channel_struct_block(MATRIX_NODE * n,int ch)
+{
+MX_STRUCT_BLOCK * ret;
+	lock_task(matrix_lock);
+	ret = _get_channel_struct_block(n,ch);
+	unlock_task(matrix_lock,"get_channel_Struct_block");
+	return ret;
+}
+
+
 void
 _set_dirty_file(MATRIX_NODE * n)
 {
+MATRIX * m;
+int i;
+	m = n->matrix;
+	for ( i = MI_SAVE_TP ; i <= MI_SAVE_BT ; i ++ ) {
+		if ( m->cal[i] && m->cal[MI_SAVE_TP]->data )
+			goto ok;
+	}
+	return;
+ok:
 	n->dirty_file_time = get_xltime();
 //	_matrix_trigger_access(n,MI_SAVE_TP);
 }
 
+void
+set_dirty_file(MATRIX_NODE *n)
+{
+	lock_task(matrix_lock);
+	_set_dirty_file(n);
+	unlock_task(matrix_lock,"set_dirty_file");
+}
+
 
 void
 _flush_all_dirty_file(MATRIX * m)
@@ -2995,7 +3220,7 @@ dc_end[0],dc_end[1],dc_end[2]);
 		}
 		err = 0;
 		n = _get_matrix_node(&err,m,dc,GN_ERROR_NORETRY,
-					dc_start[0]);
+					dc_start[0],0);
 		if ( n )
 			_matrix_trigger_access(n,_access,0);
 		_unlock_node(n,0);
@@ -3031,7 +3256,7 @@ MX_DIM_CODE_LIST * lst, * lst2, * ret;
 	err = 0;
 	if ( wait_flag )
 		n = get_matrix_node_wait(&err,m,dc,access);
-	else	n = get_matrix_node(&err,m,dc,access,t);
+	else	n = get_matrix_node(&err,m,dc,access,t,0);
 	if ( err ) {
 		if ( errp )
 			*errp = err;
@@ -3139,4 +3364,684 @@ load_indicate(int start)
 	}
 }
 
+typedef struct scan_call_work {
+	MATRIX *			m;
+	int				access;
+	INTEGER64 *			start;
+	INTEGER64 *			end;
+	int				(*func)(MATRIX_SCAN_T*);
+	void *				work;
+	
+	INTEGER64			loop_total;
+	INTEGER64			processed_total;
+	
+	unsigned			node_lock:1;
+	unsigned			ch_lock:1;
+} SCAN_CALL_WORK;
+
+int
+_scan_post_call(INTEGER64 * dim_code,SCAN_CALL_WORK * w)
+{
+MX_DIM_CODE_LIST * lst,* lst2;
+int err,ret;
+MATRIX_NODE* n;
+INTEGER64 * st_adj,* end_adj;
+int i;
+int fe;
+MATRIX_SCAN_T mst;
+	err = 0;
+	lst = get_children_list(&err,w->m,dim_code,GN_ERROR_NORETRY,1,0);
+	if ( err < 0 )
+		return err;
+	n = get_matrix_node(&err,w->m,dim_code,GN_ERROR_NORETRY,0,0);
+	if ( err < 0 )
+		return err;
+	n->scan_call[w->access/3] = 0;
+
+	matrix_trigger_access(n,w->access,1);
+
+	lock_task(matrix_lock);
+	for ( ; n->scan_call[w->access] == 0 ; ) {
+		sleep_task((int)&n->scan_call[w->access],matrix_lock);
+		lock_task(matrix_lock);
+	}
+	unlock_task(matrix_lock,"scan_post_call");
+	ret = 0;
+	if ( n->scan_call[w->access/3] < 0 ) {
+		ret = n->scan_call[w->access/3];
+		if ( ret == ME_EXIT )
+			return ret;
+	}
+	if ( w->func ) {
+		mst.dir = 0;
+		mst.dim_code = dim_code;
+		mst.n = n;
+		mst.m = n->matrix;
+		mst.work = w->work;
+		if ( (fe = (*w->func)(&mst)) < 0 ) {
+			ret = fe;
+			goto me_exit;
+		}
+	}
+	
+	st_adj = copy_dim_code(w->m,w->start);
+	end_adj = copy_dim_code(w->m,w->end);
+	for ( i = 0 ; i < w->m->p.dim ; i ++ ) {
+		st_adj[i+1] &= -(((INTEGER64)1)<<(dim_code[0]*w->m->dim_divide[i] + w->m->block_size[i]));
+		end_adj[i+1] &= -(((INTEGER64)1)<<(dim_code[0]*w->m->dim_divide[i] + w->m->block_size[i]));
+		end_adj[i+1] += (((INTEGER64)1)<<(dim_code[0]*w->m->dim_divide[i] + w->m->block_size[i]));
+	}
+	for ( lst2 = lst ; lst2 ; lst2 = lst2->next ) {
+		for ( i = 0 ; i < w->m->p.dim ; i ++ ) {
+			if ( st_adj[i+1] > lst2->dc[i+1] )
+				goto reject;
+			if ( end_adj[i+1] <= lst2->dc[i+1] )
+				goto reject;
+		}
+		err = _scan_post_call(lst2->dc,w);
+		if ( err == ME_EXIT ) {
+			ret = err;
+			break;
+		}
+		if ( err < 0 )
+			ret = err;
+	reject:
+		;
+	}
+me_exit:
+	free_mx_dim_code_list(lst);
+	d_f_ree(st_adj);
+	d_f_ree(end_adj);
+	unlock_node(n,0);
+	return ret;
+}
+
+
+int
+_scan_pre_call(INTEGER64 * dim_code,SCAN_CALL_WORK * w)
+{
+MX_DIM_CODE_LIST * lst,* lst2;
+int err,ret;
+MATRIX_NODE* n;
+INTEGER64 * st_adj,* end_adj;
+int i;
+int fe;
+MATRIX_SCAN_T mst;
+
+ss_printf("pre-call %s\n",pt_dc(w->m,dim_code,PTDC_NODE_ID));
+
+
+	err = 0;
+	lst = get_children_list(&err,w->m,dim_code,GN_ERROR_NORETRY,1,0);
+	if ( err < 0 )
+		return err;
+	n = get_matrix_node(&err,w->m,dim_code,GN_ERROR_NORETRY,0,0);
+	if ( err < 0 )
+		return err;
+	n->scan_call[w->access/3] = 0;
+
+	ret = 0;
+	st_adj = copy_dim_code(w->m,w->start);
+	end_adj = copy_dim_code(w->m,w->end);
+	for ( i = 0 ; i < w->m->p.dim ; i ++ ) {
+		st_adj[i+1] &= -(((INTEGER64)1)<<(dim_code[0]*w->m->dim_divide[i] + w->m->block_size[i]));
+		end_adj[i+1] &= -(((INTEGER64)1)<<(dim_code[0]*w->m->dim_divide[i] + w->m->block_size[i]));
+		end_adj[i+1] += (((INTEGER64)1)<<(dim_code[0]*w->m->dim_divide[i] + w->m->block_size[i]));
+	}
+	for ( lst2 = lst ; lst2 ; lst2 = lst2->next ) {
+		for ( i = 0 ; i < w->m->p.dim ; i ++ ) {
+			if ( st_adj[i+1] > lst2->dc[i+1] )
+				goto reject;
+			if ( end_adj[i+1] <= lst2->dc[i+1] )
+				goto reject;
+		}
+		err = _scan_pre_call(lst2->dc,w);
+		if ( err == ME_EXIT ) {
+			free_mx_dim_code_list(lst);
+			d_f_ree(st_adj);
+			d_f_ree(end_adj);
+			ret = err;
+			goto me_exit;
+		}
+		if ( err < 0 )
+			ret = err;
+	reject:
+		;
+	}
+	free_mx_dim_code_list(lst);
+	d_f_ree(st_adj);
+	d_f_ree(end_adj);
+
+	matrix_trigger_access(n,w->access,1);
+
+	lock_task(matrix_lock);
+	for ( ; n->scan_call[w->access/3] == 0 ; ) {
+		sleep_task((int)&n->scan_call[w->access/3],matrix_lock);
+		lock_task(matrix_lock);
+	}
+	unlock_task(matrix_lock,"scan_post_call");
+	if ( n->scan_call[w->access] < 0 ) {
+		err = n->scan_call[w->access];
+		if ( err == ME_EXIT ) {
+			ret = err;
+			goto me_exit;
+		}
+	}
+	if ( w->func ) {
+		mst.dir = 0;
+		mst.dim_code = dim_code;
+		mst.work = w->work;
+		mst.n = n;
+		mst.m = n->matrix;
+		if ( (fe = (*w->func)(&mst)) < 0 ) {
+			ret = fe;
+			goto me_exit;
+		}
+	}
+
+me_exit:
+	unlock_node(n,0);
+
+	return ret;
+}
+
+int
+matrix_scan(MATRIX * m,int dir,int access,INTEGER64 * start,INTEGER64 * end,
+	int (*func)(MATRIX_SCAN_T*),void*work)
+{
+SCAN_CALL_WORK w;
+MX_DIM_CODE_LIST * lst, * lst2;
+INTEGER64 * target;
+int f;
+INTEGER64 * st_adj,* end_adj;
+int ret;
+int err;
+int i;
+
+
+	mx_file_sync_create_node(m);
+	
+	set_matrix_env(m,"create-node","disable");
+
+	w.m = m;
+	w.access = access;
+	w.start = start;
+	w.end = end;
+	w.func = func;
+	w.work = work;
+
+	target = d_alloc(sizeof(INTEGER64)*(m->p.dim+1));
+	memset(target,0,sizeof(INTEGER64)*(m->p.dim+1));
+	target[0] = m->total_levels-1;
+
+	st_adj = copy_dim_code(w.m,w.start);
+	end_adj = copy_dim_code(w.m,w.end);
+	for ( i = 0 ; i < w.m->p.dim ; i ++ ) {
+		st_adj[i+1] &= -(((INTEGER64)1)<<(target[0]*w.m->dim_divide[i] + w.m->block_size[i]));
+		end_adj[i+1] &= -(((INTEGER64)1)<<(target[0]*w.m->dim_divide[i] + w.m->block_size[i]));
+		end_adj[i+1] += (((INTEGER64)1)<<(target[0]*w.m->dim_divide[i] + w.m->block_size[i]));
+	}
+
+	ret = 0;
+	f = 0;
+	for ( ; ; ) {
+		lst = mx_search_level_node(m,target,100);
+		if ( lst == 0 ) {
+			break;
+		}
+		if ( f && cmp_dim_code(m,target,lst->dc) == 0 ) {
+			lst2 = lst;
+			lst = lst->next;
+			d_f_ree(lst2->dc);
+			d_f_ree(lst2);
+			if ( lst == 0 )
+				break;
+		}
+		f = 1;
+
+		for ( ; lst ; ) {
+			lst2 = lst->next;
+
+			memcpy(target,lst->dc,
+				sizeof(INTEGER64)*(m->p.dim+1));
+
+			for ( i = 0 ; i < w.m->p.dim ; i ++ ) {
+				if ( st_adj[i+1] > lst->dc[i+1] )
+					goto reject;
+				if ( end_adj[i+1] <= lst->dc[i+1] )
+					goto reject;
+			}
+			if ( dir == 0 )
+				err = _scan_pre_call(lst->dc,&w);
+			else	err = _scan_post_call(lst->dc,&w);
+			if ( err < 0 )
+				ret = err;
+
+		reject:
+			d_f_ree(lst->dc);
+			d_f_ree(lst);
+
+			lst = lst2;
+
+		}
+	}
+	d_f_ree(target);
+	d_f_ree(st_adj);
+	d_f_ree(end_adj);
+	
+	return ret;
+}
+
+INTEGER64
+get_loop_amount(MATRIX * m,INTEGER64 * start,INTEGER64 * end,INTEGER64 * dc)
+{
+INTEGER64 s;
+int level,i;
+INTEGER64 ret,sq,spn_start,spn_end;
+INTEGER64 *_end,*_start;
+	_start = copy_dim_code(m,dc);
+	_end = copy_dim_code(m,dc);
+	for ( i = 0 ; i < m->p.dim ; i ++ ) {
+		s = ((INTEGER64)1)<<(dc[0]*m->dim_divide[i] + m->block_size[i]);
+		_start[i+1] &= -s;
+		if ( _start[i+1] < start[i+1] )
+			_start[i+1] = start[i+1];
+		_end[i+1] &= -s;
+		_end[i+1] += s;
+		if ( _end[i+1] > end[i+1] )
+			_end[i+1] = end[i+1];
+	}
+	ret = 0;
+	for ( level = dc[0] ; level >= 0 ; level -- ) {
+		sq = 1;
+		for ( i = 0 ; i < m->p.dim ; i ++ ) {
+			s = ((INTEGER64)1)<<(level*m->dim_divide[i] + m->block_size[i]);
+			spn_start = _start[i+1] & (-s);
+			spn_end = _end[i+1] & (-s);
+			if ( _end[i+1] & (s-1) )
+				spn_end += s;
+			spn_end -= spn_start;
+			spn_end >>= level * m->dim_divide[i];
+			
+			sq *= spn_end;
+		}
+		ret += sq;
+	}
+ss_printf("GLA %lli\n",ret);
+	return ret;
+}
+
+int
+_scan_call_loading(INTEGER64 * dim_code,SCAN_CALL_WORK * w,MATRIX_SCAN_T * mst)
+{
+int err;
+MATRIX_NODE * n;
+	switch ( mst->loading_type ) {
+	case SLT_NODE:
+		err = 0;
+		n = get_matrix_node_wait(&err,w->m,dim_code,mst->gn_create);
+		mst->n = n;
+		mst->channel = 0;
+		if ( err < 0 )
+			return err;
+		w->node_lock = 1;
+		return 0;
+	case SLT_CHANNEL:
+		err = 0;
+		n = get_matrix_node_wait(&err,w->m,dim_code,mst->gn_create);
+		if ( err < 0 )
+			return err;
+		unlock_node(n,0);
+		mst->channel_data = get_matrix_node_channel(&err,&n,w->m,dim_code,mst->channel,
+					mst->gn_tree_node,mst->gn_create,0);
+		mst->n = n;
+		if ( err < 0 )
+			return err;
+		w->node_lock = 1;
+		w->ch_lock = 1;
+		return 0;
+	default:
+		er_panic("_scan_call_loading");
+		return 0;
+	}
+}
+
+
+int
+_scan_call_force(INTEGER64 * dim_code,SCAN_CALL_WORK * w)
+{
+int err,ret;
+//MATRIX_NODE* n;
+INTEGER64 * st_adj,* end_adj;
+INTEGER64 * dc_adj,*ix,*inc;
+int i;
+int fe,fe1;
+MATRIX_SCAN_T mst;
+unsigned channel_lock;
+unsigned node_lock;
+int proc_total;
+static INTEGER64 ind_time;
+
+ss_printf("call-force %s %s %s\n",pt_dc(w->m,dim_code,PTDC_NODE_ID),
+pt_dc(w->m,w->start,PTDC_NODE_ID),pt_dc(w->m,w->end,PTDC_NODE_ID));
+
+	ret = 0;
+	st_adj = end_adj = 0;
+	dc_adj = ix = inc = 0;
+	mst.dim_code = dim_code;
+	mst.n = 0;
+	mst.m = w->m;
+	mst.work = w->work;
+	mst.dir = MST_FIRST;
+	fe1 = 0;
+	channel_lock = node_lock = 0;
+	if ( w->func ) {
+		if ( (fe1 = fe = (*w->func)(&mst)) < 0 ) {
+			ret = fe;
+			return ret;
+		}
+	}
+
+	if ( fe1 & MST_1LOAD ) {
+		w->node_lock = w->ch_lock = 0;
+		err = _scan_call_loading(dim_code,w,&mst);
+ss_printf("ERR? %i\n",err);
+		if ( err < 0 )
+			return err;
+		if ( w->node_lock )
+			node_lock ++;
+		if ( w->ch_lock )
+			channel_lock ++;
+	}
+	
+	mst.dir = MST_AFTERLOAD;
+	if ( w->func ) {
+		if ( (fe = (*w->func)(&mst)) < 0 ) {
+			ret = fe;
+			goto me_exit;
+		}
+	}
+	
+	if ( dim_code[0] > 0 && (fe1 & MST_SRCH) ) {
+	INTEGER64 level;
+		st_adj = copy_dim_code(w->m,w->start);
+		end_adj = copy_dim_code(w->m,w->end);
+		dc_adj = copy_dim_code(w->m,dim_code);
+		ix = copy_dim_code(w->m,dim_code);
+		inc = d_alloc(sizeof(INTEGER64)*(w->m->p.dim+1));
+		inc[0] = dim_code[0];
+		inc[0] --;
+		ix[0] --;
+		st_adj[0] --;
+		end_adj[0] --;
+		level = dim_code[0]-1;
+		for ( i = 0 ; i < w->m->p.dim ; i ++ ) {
+			st_adj[i+1] &= -(((INTEGER64)1)<<(level*w->m->dim_divide[i] + w->m->block_size[i]));
+			end_adj[i+1] &= -(((INTEGER64)1)<<(level*w->m->dim_divide[i] + w->m->block_size[i]));
+			end_adj[i+1] += (((INTEGER64)1)<<(level*w->m->dim_divide[i] + w->m->block_size[i]));
+
+			dc_adj[i+1] &= -(((INTEGER64)1)<<(dim_code[0]*w->m->dim_divide[i] + w->m->block_size[i]));
+			if ( st_adj[i+1] < dc_adj[i+1] )
+				st_adj[i+1] = dc_adj[i+1];
+			dc_adj[i+1] += (((INTEGER64)1)<<(dim_code[0]*w->m->dim_divide[i] + w->m->block_size[i]));
+			if ( end_adj[i+1] > dc_adj[i+1] )
+				end_adj[i+1] = dc_adj[i+1];
+			ix[i+1] = st_adj[i+1];
+			inc[i+1] = (((INTEGER64)1)<<((level)*w->m->dim_divide[i] + w->m->block_size[i]));
+		}
+		for ( ; ; ) {
+			err = _scan_call_force(ix,w);
+			if ( err < 0 ) {
+				ret = err;
+				break;
+			}
+			if ( inc_dim_code_ix(ix,st_adj,inc,end_adj,w->m->p.dim) )
+				break;
+		}
+		proc_total = 0;
+	}
+	else {
+		proc_total = -1;
+	}
+
+	mst.dir = MST_AFTERSRCH;
+	if ( w->func ) {
+		if ( (fe = (*w->func)(&mst)) < 0 ) {
+			ret = fe;
+			goto me_exit;
+		}
+	}
+
+	if ( fe1 & MST_2LOAD ) {
+		w->node_lock = w->ch_lock = 0;
+		err = _scan_call_loading(dim_code,w,&mst);
+		if ( err < 0 )
+			return err;
+		if ( w->node_lock )
+			node_lock ++;
+		if ( w->ch_lock )
+			channel_lock ++;
+	}
+	
+	mst.dir = MST_LAST;
+	if ( w->func ) {
+		if ( (fe = (*w->func)(&mst)) < 0 ) {
+			ret = fe;
+			goto me_exit;
+		}
+	}
+
+	if ( proc_total < 0 ) {
+		w->processed_total += get_loop_amount(w->m,w->start,w->end,dim_code);
+	}
+	else {
+	INTEGER64 sq;
+		sq = 1;
+		for ( i = 0 ; i < w->m->p.dim ; i ++ )
+			sq *= ((INTEGER64)1)<<w->m->block_size[i];
+		w->processed_total += sq;
+	}
+if ( ind_time != get_xltime() ) {
+ss_printf("TOTAL = %lli / %lli %f\n",w->processed_total,w->loop_total,
+100.0 * w->processed_total / w->loop_total);
+ind_time = get_xltime();
+}
+
+me_exit:
+	if ( st_adj )
+		d_f_ree(st_adj);
+	if ( end_adj )
+		d_f_ree(end_adj);
+	if ( ix )
+		d_f_ree(ix);
+	if ( dc_adj )
+		d_f_ree(dc_adj);
+	if ( inc )
+		d_f_ree(inc);
+	if ( mst.n ) {
+		for ( ; channel_lock ; channel_lock -- )
+			matrix_node_channel_unlock(mst.n,mst.dirty_flags);
+		for ( ; node_lock ; node_lock -- )
+			unlock_node(mst.n,0);
+	}
+	return ret;
+}
+
+
+
+
+int
+matrix_scan_force(MATRIX * m,int access,INTEGER64 * start,INTEGER64 * end,
+		int (*func)(MATRIX_SCAN_T*),void*work)
+{
+SCAN_CALL_WORK w;
+INTEGER64 * target;
+int ret;
+int err;
+
+
+//	mx_file_sync_create_node(m);
+	
+	set_matrix_env(m,"create-node","enable");
+
+	w.m = m;
+	w.access = access;
+	w.start = start;
+	w.end = end;
+	w.func = func;
+	w.work = work;
+
+
+	target = d_alloc(sizeof(INTEGER64)*(m->p.dim+1));
+	memset(target,0,sizeof(INTEGER64)*(m->p.dim+1));
+	target[0] = m->total_levels-1;
+
+	w.loop_total = get_loop_amount(m,start,end,target);
+	w.processed_total = 0;
+
+	err = _scan_call_force(target,&w);
+	d_f_ree(target);
+	
+	return ret;
+}
+
+int
+decode_matrix_node(MATRIX_NODE * n,XL_SEXP * s)
+{
+int i;
+unsigned char nlist_type;
+unsigned char * ptr,* start;
+int size;
+INTEGER64 nlist_dim_bit_field_len;
+INTEGER64 nlist_dim_addr_len;
+MATRIX * m;
+INTEGER64 channel_no,data_size;
+MATRIX_DATA_TYPE * tp;
+int er_data;
+
+	if ( get_type(s) != XLT_RAW )
+		return -1;
+	lock_task(matrix_lock);
+	_lock_node(n,0);
+	m = n->matrix;
+	size = s->raw.size;
+	start = ptr =(unsigned char*) s->raw.data;
+	nlist_type = (unsigned char)*ptr;
+	ptr++;
+	if ( nlist_type ) {
+		ptr = get_uncompressed_code64(&nlist_dim_bit_field_len,ptr);
+		if ( n->nlist_dim_bit_field )
+			d_f_ree(n->nlist_dim_bit_field);
+		n->nlist_dim_bit_field = d_alloc(nlist_dim_bit_field_len);
+		memcpy(n->nlist_dim_bit_field,ptr,nlist_dim_bit_field_len);
+		n->nlist_dim_bit_field_len = nlist_dim_bit_field_len;
+		_free_nlist_dim_addr(n->nlist_dim_addr,n->nlist_dim_addr_len);
+		n->nlist_dim_addr_len = 0;
+		
+		ptr += nlist_dim_bit_field_len;
+	}
+	else {
+		if ( n->nlist_dim_addr )
+			_free_nlist_dim_addr(n->nlist_dim_addr,n->nlist_dim_addr_len);
+		ptr = get_uncompressed_code64(&nlist_dim_addr_len,ptr);
+		n->nlist_dim_addr_len = nlist_dim_addr_len;
+		n->nlist_dim_addr = d_alloc(sizeof(INTEGER64*)*nlist_dim_addr_len);
+		for ( i = 0 ; i < nlist_dim_addr_len ; i ++ ) {
+			n->nlist_dim_addr[i] = d_alloc(sizeof(INTEGER64)*(m->p.dim+1));
+			ptr = get_uncompressed_dim_code(n->nlist_dim_addr[i],ptr);
+		}
+	}
+	for ( ; ptr - start < size ; ) {
+		er_data = 0;
+		ptr = get_uncompressed_code64(&channel_no,ptr);
+
+		if ( channel_no < 0 )
+			er_data = 1;
+		if ( channel_no >= m->p.channel_nos )
+			er_data = 1;
+		ptr = get_uncompressed_code64(&data_size,ptr);
+		if ( er_data == 0 ) {
+			tp = m->channel_info[channel_no].data_type;
+			if ( n->channel[channel_no].data ) {
+				(*tp->free_data)(tp,n->channel[channel_no].data);
+			}
+			n->channel[channel_no].data = (*tp->convert_to_host)
+				(tp,ptr,(int)data_size,MD_DALLOC,0,__FILE__,__LINE__);
+		}
+		ptr += data_size;
+	}
+	_unlock_node(n,0);
+	unlock_task(matrix_lock,"");
+	return 0;
+}
+
+XL_SEXP *
+encode_matrix_node(MATRIX_NODE * n)
+{
+RECORD_LIST64 * rl;
+unsigned char nlist_len[sizeof(INTEGER64)+2];
+unsigned char nlist_type;
+unsigned char * ptr;
+MATRIX * m;
+int i;
+unsigned char * dim_code_buffer;
+unsigned char * channel_buffer;
+MATRIX_DATA_TYPE * tp;
+int size;
+XL_SEXP * ret;
+CHAIN_LIST64 * rl_ptr;
+
+
+	rl = new_recordlist64(0,0);
+	m = n->matrix;
+	if (  m->dim_bit_field ) {
+		nlist_type = 1;
+		set_recordlist_chain64(rl,&nlist_type,1,0);
+		ptr = get_compressed_code64(nlist_len,n->nlist_dim_bit_field_len);
+		set_recordlist_chain64(rl,nlist_len,ptr - &nlist_len[0],0);
+		set_recordlist_chain64(rl,n->nlist_dim_bit_field,n->nlist_dim_bit_field_len,0);
+	}
+	else {
+		nlist_type = 0;
+		set_recordlist_chain64(rl,&nlist_type,1,0);
+		ptr = get_compressed_code64(nlist_len,n->nlist_dim_addr_len);
+		set_recordlist_chain64(rl,nlist_len,ptr - &nlist_len[0],0);
+		for ( i = 0 ; i < n->nlist_dim_addr_len ; i ++ ) {
+			dim_code_buffer = d_alloc(sizeof(INTEGER64)*2*(m->p.dim+1));
+			ptr = get_compressed_dim_code(dim_code_buffer,n->nlist_dim_addr[i],m->p.dim);
+			set_recordlist_chain64(rl,dim_code_buffer,ptr - dim_code_buffer,1);
+		}
+	}
+	for ( i = 0 ; i < m->p.channel_nos ; i ++ ) {
+		tp = m->channel_info[i].data_type;
+		if ( tp == 0 )
+			continue;
+		if ( (m->channel_info[i].flags & MF_SEND) == 0 )
+			continue;
+		if ( n->channel[i].data == 0 )
+			continue;
+		channel_buffer = d_alloc(sizeof(INTEGER64)*2);
+		ptr = get_compressed_code64(channel_buffer,i);
+		set_recordlist_chain64(rl,channel_buffer,ptr - channel_buffer,1);
+		rl_ptr = rl->chain_tail;
+		(*tp->convert_to_net)(tp,rl,n->channel[i].data);
+		set_recordlist_code(rl,rl_ptr);
+	}
+	size = setup_recordlist64(rl);
+	ret = get_raw(rl->data,size);
+	free_recordlist64(rl);
+	return ret;
+}
+
+
+void
+set_dirty_file_sb(MX_STRUCT_BLOCK * b)
+{
+	lock_task(matrix_lock);
+	if ( b->n.m->channel_info[b->n.channel].flags & MF_FILE )
+		_set_dirty_file(b->n.n);
+	unlock_task(matrix_lock,"set_dirty_file_sb");
+}
+
+
 
