@@ -1,4 +1,4 @@
-/* ver.B.b11(BLAZE) */
+/* ver.B.b14.01(BLAZE) */
 
 
 #include "version.h"
@@ -7,9 +7,9 @@
 VERSION ap_version = {
 	"AP.code",
 	"gpstester",
-	"(C) 2000-2005 Hirohisa Mori",
-	"ver.B.b11(BLAZE)",
-	2006,7,26,
+	"(C) 2000-2007 Hirohisa Mori",
+	"ver.B.b14.01(BLAZE)",
+	2007,2,27,
 	0,
 	"gps I/O tester ",
 };
