@@ -1,4 +1,4 @@
-/* ver.B.b14.01(BLAZE) */
+/* ver.B.b14.02(BLAZE) */
 
 
 #include "version.h"
@@ -8,9 +8,9 @@ VERSION ap_version = {
 	"AP.code",
 	"xlsv",
 	"(C) 2000-2007 Hirohisa Mori",
-	"ver.B.b14.01(BLAZE)",
+	"ver.B.b14.02(BLAZE)",
 	2007,2,27,
 	0,
-	"XL server agent ? gbs/env/proj/unix/MacOSX/lib/pkgconfig",
+	"XL server agent ? gbs/doc/xml/man/html/ja-cosmos",
 };
 
