@@ -1,4 +1,4 @@
-/* ver.B.b14(BLAZE) */
+/* ver.B.b14.02(BLAZE) */
 
 
 #include "version.h"
@@ -8,8 +8,8 @@ VERSION machine_code_version = {
 	"MA.code",
 	"posix.linux.Interface",
 	"(C) 2000-2007 Hirohisa Mori",
-	"ver.B.b14(BLAZE)",
-	2007,2,13,
+	"ver.B.b14.02(BLAZE)",
+	2007,2,28,
 	0,
 	"machine (posix.linux) dependent library ",
 };
