@@ -1,4 +1,4 @@
-/* ver.B.b11.01(BLAZE) */
+/* ver.B.b14(BLAZE) */
 
 
 #include "version.h"
@@ -7,9 +7,9 @@
 VERSION machine_code_version = {
 	"MA.code",
 	"win.xp.Interface",
-	"(C) 2000-2005 Hirohisa Mori",
-	"ver.B.b11.01(BLAZE)",
-	2006,8,22,
+	"(C) 2000-2007 Hirohisa Mori",
+	"ver.B.b14(BLAZE)",
+	2007,2,27,
 	0,
 	"machine (win.xp) dependent library ",
 };
