@@ -1,4 +1,4 @@
-/* ver.B.b11(BLAZE) */
+/* ver.B.b14.02(BLAZE) */
 
 
 #include "version.h"
@@ -7,9 +7,9 @@
 VERSION ap_version = {
 	"AP.code",
 	"mxmap",
-	"(C) 2000-2005 Hirohisa Mori",
-	"ver.B.b11(BLAZE)",
-	2006,7,26,
+	"(C) 2000-2007 Hirohisa Mori",
+	"ver.B.b14.02(BLAZE)",
+	2007,2,28,
 	0,
 	"make matrix map ",
 };
