@@ -1,4 +1,4 @@
-/* ver.B.b11.02(BLAZE) */
+/* ver.B.b13(BLAZE) */
 
 
 #include "version.h"
@@ -8,8 +8,8 @@ VERSION ap_version = {
 	"AP.code",
 	"vtester",
 	"(C) 2000-2005 Hirohisa Mori",
-	"ver.B.b11.02(BLAZE)",
-	2006,8,21,
+	"ver.B.b13(BLAZE)",
+	2006,10,17,
 	0,
 	"VObject tester ",
 };
