@@ -1,4 +1,4 @@
-/* ver.B.b14(BLAZE) */
+/* ver.B.b14.02(BLAZE) */
 
 
 #include "version.h"
@@ -8,8 +8,8 @@ VERSION xl_code_version = {
 	"XL.code",
 	"XL.lib",
 	"(C) 2000-2007 Hirohisa Mori",
-	"ver.B.b14(BLAZE)",
-	2007,2,10,
+	"ver.B.b14.02(BLAZE)",
+	2007,2,27,
 	0,
 	"XL protocol library ",
 };
