@@ -23,6 +23,7 @@
 #include <stdlib.h>
 
 #include "v/VObject.h"
+#include "v/VWindow.h"
 #include "v/vobj_utils.h"
 #include "v/VMarshaler.h"
 #include "v/VMacro.h"
@@ -48,13 +49,15 @@ VSysArgTranslator * vsat_list;
 void
 gc_vobj_utils_lock()
 {
-	lock_task(vobj_utils_lock);
+	if ( lock_up_test(vobj_utils_lock) )
+		lock_task(vobj_utils_lock);
 }
 
 void
 gc_vobj_utils_unlock()
 {
-	unlock_task(vobj_utils_lock,"unlock");
+	if ( lock_up_test(vobj_utils_lock) )
+		unlock_task(vobj_utils_lock,"unlock");
 }
 
 void
@@ -1021,6 +1024,9 @@ get_sts_from_sf(XLISP_ENV * arg_env,VObjectStatus *s,int flags, XL_SYM_FIELD *sf
 		
 	GET_ATTR_AND_SET_I(homogeneous, VSF_HOMOGEN);
 	GET_ATTR_AND_SET_SIZE(spacing, VSF_SPACING);
+	
+	if ( get_sf_attribute(sf,l_string(std_cm,"focus")) )
+		f |= VSF_FOCUS;
 
 	if ( (flags & VSF_ALIGN) == 0 )
 		s->alignv = s->alignh = VALIGN_EXPAND;
@@ -1339,4 +1345,168 @@ int _v,_h;
 	}
 }
 
+
+VWindow *
+get_window_object(VObject * obj)
+{
+VObjectStatus sts;
+	for ( ; obj->get_type() != VO_WIND ; ) {
+		if ( obj->get_status(&sts,VSF_PARENT).code != V_ER_NO_ERR )
+			return 0;
+		obj = sts.parent;
+		if ( obj == 0 )
+			return 0;
+	}
+	return dynamic_cast<VWindow*>(obj);
+}
+
+typedef struct modal_event_func_tbl {
+	int			type;
+	char *			name;
+} MODAL_EVENT_FUNC_TBL;
+
+typedef struct modal_event_arg {
+	DIALOG_IO *		io;
+	int			type;
+} MODAL_EVENT_ARG;
+
+MODAL_EVENT_FUNC_TBL modal_event_func_table[] = {
+	{DIO_OK,"ok"},
+	{DIO_CANCEL,"cancel"},
+	{DIO_SAVE,"save"},
+	{DIO_DONTSAVE,"dontsave"},
+	{0,0}
+};
+
+V_CALLBACK_D(v_modal_dialog_handler)
+{
+MODAL_EVENT_ARG * ag;
+	ag = (MODAL_EVENT_ARG*)user_arg;
+	ag->io->ret = ag->type;
+	lock_task(vobj_utils_lock);
+	wakeup_task((int)ag->io);
+	unlock_task(vobj_utils_lock,"modal");
+}
+
+
+int modal_flag;
+
+void
+v_modal_dialog(DIALOG_IO * io)
+{
+XL_SEXP * call,*ret;
+XL_SEXP * id;
+int i;
+char buf[10];
+VObject * obj;
+VObjectStatus _sts;
+MODAL_EVENT_ARG * ag;
+	gc_push(0,0,"v_modal_dialog");
+	call = cons(n_get_symbol("param"),0);
+	for ( i = 0 ; i < DIO_MSG_LEN ; i ++ ) {
+		if ( io->msg[i] == 0 )
+			continue;
+		sprintf(buf,"msg%i",i);
+		call = cons(List(n_get_symbol(buf),get_string(io->msg[i]),-1),call);
+	}
+	call = reverse(call);
+	call = List(n_get_symbol(io->type),List(n_get_symbol("quote"),call,-1),-1);
+
+	io->ret = DIO_NONE;
+	ret = eval(vobj_env,call);
+	if ( get_type(ret) == XLT_ERROR ) {
+		print_sexp(s_stdout,ret,0);
+		ss_printf("\n");
+		io->ret = DIO_ERR;
+		goto end;
+	}
+	ag = (MODAL_EVENT_ARG*)d_alloc(sizeof(*ag)*DIO_MAX);
+	memset(ag,0,sizeof(*ag)*DIO_MAX);
+	for ( i = 0 ; modal_event_func_table[i].name ; i ++ ) {
+		id = vobj_get_object_by_name(ret,l_string(std_cm,modal_event_func_table[i].name),0);
+		if ( get_type(id) != XLT_INTEGER )
+			continue;
+		obj = VObject::get_object_by_id(id->integer.data);
+		if ( obj == 0 )
+			continue;
+		ag[i].type = modal_event_func_table[i].type;
+		ag[i].io = io;
+		_sts.value_event_handler = v_modal_dialog_handler;
+		_sts.value_eh_arg = (void*)&ag[i];
+		obj->set_status(&_sts,VSF_VALUE_EH);
+	}
+	id = vobj_get_object_by_name(ret,l_string(std_cm,"window"),0);
+	if ( get_type(id) != XLT_INTEGER ) {
+		print_sexp(s_stdout,id,0);
+		ss_printf("\n");
+		io->ret = DIO_ERR;
+		goto end;
+	}
+	obj = VObject::get_object_by_id(id->integer.data);
+	if ( obj == 0 ) {
+		ss_printf("v_modal_dialog obj error\n");
+		goto end;
+	}
+	lock_task(vobj_utils_lock);
+	_sts.visible = 1;
+	obj->set_status(&_sts,VSF_VISIBLE);
+	modal_flag = 1;
+	wakeup_task((int)&modal_flag);
+	for ( ; io->ret == DIO_NONE ; ) {
+		sleep_task((int)io,vobj_utils_lock);
+		lock_task(vobj_utils_lock);
+	}
+	_sts.visible = 0;
+	obj->set_status(&_sts,VSF_VISIBLE);
+	unlock_task(vobj_utils_lock,"modal");
+	obj->destroy();
+	d_f_ree(ag);
+end:
+	gc_pop(0,0);
+}
+
+
+int
+reset_modal_flag(int wait_flag)
+{
+int ret;
+	lock_task(vobj_utils_lock);
+	if ( wait_flag ) {
+		for ( ; modal_flag == 0 ; ) {
+			sleep_task((int)&modal_flag,vobj_utils_lock);
+			lock_task(vobj_utils_lock);
+		}
+	}
+	ret = modal_flag;
+	modal_flag = 0;
+	unlock_task(vobj_utils_lock,"reset_modal_flag");
+	return ret;
+}
+
+
+void
+modal_quit_task()
+{
+DIALOG_IO io;
+bool vobject_quit();
+XL_INTERPRETER * xli;
+extern int (*vobject_quit_callback)();
+
+
+	xli = new_xl_interpreter();
+	xli->a_type = XLA_SELF;
+	setup_i(xli);
+
+	VWindow::close_all_window();
+
+	memset(&io,0,sizeof(io));
+	io.type = "modal-quit-application";
+	v_modal_dialog(&io);
+	vobject_quit_callback = 0;
+	vobject_quit();
+
+}
+
+
+
 } // extern "C"
