@@ -42,7 +42,7 @@ int check_unit_string(L_CHAR * str);
 int ttest;
 SEM lex_lock;
 
-#define TEST_LEN	1000
+#define TEST_LEN	10000
 
 char * emp_tag_table[] = {
 	"meta",
@@ -71,9 +71,9 @@ test_mem(char * str)
 {
 int i;
 char ** d;
-/*
+
 	fprintf(stderr,"%s...\n",str);
-*/
+
 	fflush(stderr);
 	d = d_alloc(TEST_LEN*sizeof(char*));
 	for ( i = 0 ; i < TEST_LEN ; i ++ )
@@ -81,9 +81,9 @@ char ** d;
 	for ( i = 0 ; i < TEST_LEN ; i ++ )
 		d_f_ree(d[i]);
 	d_f_ree(d);
-/*
+
 	fprintf(stderr,"%s...END\n",str);
-*/
+
 }
 
 
@@ -1368,6 +1368,17 @@ entity_error:
 	return 0;
 }
 
+void
+xl_set_cm(XL_SEXP * s,CODE_METHOD * cm)
+{
+XL_FILE * f;
+	f = s->h.file;
+	(*f->cm->close)(0,f->cm_work);
+	f->cm = cm;
+	f->cm_work = (*f->cm->open)();
+	s_set_cm(f->st,cm);
+}
+
 
 int
 _gb_lex(XL_FILE * f,int ent_mode)
