@@ -260,6 +260,7 @@ XL_SEXP * ret;
 XL_SEXP * ptr;
 HTTP_AGENT_LIST * a_target;
 
+
 	if ( aa->alias_root )
 		a_target = aa->alias_root;
 	else	a_target = aa;
@@ -295,6 +296,9 @@ HTTP_AGENT_LIST * a_target;
 				-1));
 	if ( get_type(ret) == XLT_ERROR ) {
 		close_interpreter(a_target->iid);
+		ss_printf("INIT ERR");
+		print_sexp(s_stdout,ret,0);
+		ss_printf("\n");
 		_invalid_all(a_target);
 		return;
 	}
@@ -430,7 +434,6 @@ L_CHAR * cc;
 				s_set_cm(st,cm);
 			}
 		prt:
-
 			print_sexp(st,data,
 				PF_MULTI_ROOT|PF_XML|PF_INDENT);
 		}
@@ -463,6 +466,42 @@ sys_err:
 }
 
 
+char *
+convert_percent_reverse(char * inp)
+{
+char * ret;
+int len;
+char * p,*q;
+char ch1,ch2;
+
+	len = strlen(inp);
+	ret = d_alloc(len*5 + 1);
+	p = inp;
+	q = ret;
+	for ( ; *p ; p ++ ) {
+		if ( '0' <= *p && *p <= '9' )
+			*q ++ = *p;
+		else if ( 'a' <= *p && *p <= 'z' )
+			*q ++ = *p;
+		else if ( 'A' <= *p && *p <= 'Z' )
+			*q ++ = *p;
+		else {
+			*q ++ = '%';
+			ch1 = ((*p)>>4)&0xf;
+			ch2 = (*p) & 0xf;
+			if ( ch1 < 10 )
+				*q ++ = ch1 + '0';
+			else	*q ++ = ch1 - 10 + 'A';
+			if ( ch2 < 10 )
+				*q ++ = ch2 + '0';
+			else	*q ++ = ch2 - 10 + 'A';
+		}
+	}
+	*q = 0;
+	return ret;
+}
+
+
 char *
 convert_percent(char * inp,char * limited_list)
 {
